<?xml version='1.0'?>
<xsl:stylesheet version="1.0"    
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" >

<!-- (c) 2013, Trimble Navigation Limited. All rights reserved.                                -->
<!-- Permission is hereby granted to use, copy, modify, or distribute this style sheet for any -->
<!-- purpose and without fee, provided that the above copyright notice appears in all copies   -->
<!-- and that both the copyright notice and the limited warranty and restricted rights notice  -->
<!-- below appear in all supporting documentation.                                             -->

<!-- TRIMBLE NAVIGATION LIMITED PROVIDES THIS STYLE SHEET "AS IS" AND WITH ALL FAULTS.         -->
<!-- TRIMBLE NAVIGATION LIMITED SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY -->
<!-- OR FITNESS FOR A PARTICULAR USE. TRIMBLE NAVIGATION LIMITED DOES NOT WARRANT THAT THE     -->
<!-- OPERATION OF THIS STYLE SHEET WILL BE UNINTERRUPTED OR ERROR FREE.                        -->

<xsl:output method="text" omit-xml-declaration="yes" encoding="ISO-8859-1"/>

<!-- Set the numeric display details i.e. decimal point, thousands separator etc -->
<xsl:variable name="DecPt" select="string('.')"/>    <!-- Change as appropriate for US/European -->
<xsl:variable name="GroupSep" select="string(',')"/> <!-- Change as appropriate for US/European -->
<!-- Also change decimal-separator & grouping-separator in decimal-format below 
     as appropriate for US/European output -->
<xsl:decimal-format name="Standard" 
                    decimal-separator="."
                    grouping-separator=","
                    infinity="Infinity"
                    minus-sign="-"
                    NaN="?"
                    percent="%"
                    per-mille="&#2030;"
                    zero-digit="0" 
                    digit="#" 
                    pattern-separator=";" />

<xsl:variable name="DecPl0" select="string('#0')"/>
<xsl:variable name="DecPl1" select="string(concat('#0', $DecPt, '0'))"/>
<xsl:variable name="DecPl2" select="string(concat('#0', $DecPt, '00'))"/>
<xsl:variable name="DecPl3" select="string(concat('#0', $DecPt, '000'))"/>
<xsl:variable name="DecPl4" select="string(concat('#0', $DecPt, '0000'))"/>
<xsl:variable name="DecPl5" select="string(concat('#0', $DecPt, '00000'))"/>
<xsl:variable name="DecPl6" select="string(concat('#0', $DecPt, '000000'))"/>
<xsl:variable name="DecPl8" select="string(concat('#0', $DecPt, '00000000'))"/>

<xsl:variable name="fileExt" select="'kof'"/>

<!-- Define search keys to speed up searchs -->
<xsl:key name="reducedPt-search" match="/JOBFile/Reductions/Point" use="Name"/>
<xsl:key name="tgtID-search" match="/JOBFile/FieldBook/TargetRecord" use="@ID"/>
<xsl:key name="antID-search" match="/JOBFile/FieldBook/AntennaRecord" use="@ID"/>
<xsl:key name="instID-search" match="/JOBFile/FieldBook/InstrumentRecord" use="@ID"/>

<!-- **************************************************************** -->
<!-- Set global variables from the Environment section of JobXML file -->
<!-- **************************************************************** -->
<xsl:variable name="DistUnit"   select="/JOBFile/Environment/DisplaySettings/DistanceUnits" />
<xsl:variable name="AngleUnit"  select="/JOBFile/Environment/DisplaySettings/AngleUnits" />
<xsl:variable name="CoordOrder" select="/JOBFile/Environment/DisplaySettings/CoordinateOrder" />
<xsl:variable name="TempUnit"   select="/JOBFile/Environment/DisplaySettings/TemperatureUnits" />
<xsl:variable name="PressUnit"  select="/JOBFile/Environment/DisplaySettings/PressureUnits" />

<!-- Setup conversion factor for coordinate and distance values -->
<!-- Dist/coord values in JobXML file are always in metres -->
<xsl:variable name="DistConvFactor">
  <xsl:choose>
    <xsl:when test="$DistUnit='Metres'">1.0</xsl:when>
    <xsl:when test="$DistUnit='InternationalFeet'">3.280839895</xsl:when>
    <xsl:when test="$DistUnit='USSurveyFeet'">3.2808333333357</xsl:when>
    <xsl:otherwise>1.0</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup conversion factor for angular values -->
<!-- Angular values in JobXML file are always in decimal degrees -->
<xsl:variable name="AngleConvFactor">
  <xsl:choose>
    <xsl:when test="$AngleUnit='DMSDegrees'">1.0</xsl:when>
    <xsl:when test="$AngleUnit='Gons'">1.111111111111</xsl:when>
    <xsl:when test="$AngleUnit='Mils'">17.77777777777</xsl:when>
    <xsl:otherwise>1.0</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup boolean variable for coordinate order -->
<xsl:variable name="NECoords">
  <xsl:choose>
    <xsl:when test="$CoordOrder='North-East-Elevation'">true</xsl:when>
    <xsl:when test="$CoordOrder='X-Y-Z'">true</xsl:when>
    <xsl:otherwise>false</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup conversion factor for pressure values -->
<!-- Pressure values in JobXML file are always in millibars (hPa) -->
<xsl:variable name="PressConvFactor">
  <xsl:choose>
    <xsl:when test="$PressUnit='MilliBar'">1.0</xsl:when>
    <xsl:when test="$PressUnit='InchHg'">0.029529921</xsl:when>
    <xsl:when test="$PressUnit='mmHg'">0.75006</xsl:when>
    <xsl:otherwise>1.0</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:variable name="product">
  <xsl:choose>
    <xsl:when test="JOBFile/@product"><xsl:value-of select="JOBFile/@product"/></xsl:when>
    <xsl:otherwise>Trimble Survey Controller</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:variable name="WGS84SemiMajorAxis" select="6378137.000"/>
<xsl:variable name="WGS84Flattening" select="298.257222101"/>
<xsl:variable name="WGS84EccentricitySquared" select="0.00669438002290"/>
<xsl:variable name="Pi" select="3.14159265358979323846264"/>
<xsl:variable name="halfPi" select="$Pi div 2.0"/>

<!-- **************************************************************** -->
<!-- ************************** Main Loop *************************** -->
<!-- **************************************************************** -->
<xsl:template match="/" >

  <xsl:call-template name="FileHeader"/>

  <!-- Select the FieldBook node to process -->
  <xsl:apply-templates select="JOBFile/FieldBook" />

</xsl:template>


<!-- **************************************************************** -->
<!-- ***************** FieldBook Node Processing ******************** -->
<!-- **************************************************************** -->
<xsl:template match="FieldBook">
<!-- Process the records under the FieldBook node in the order encountered -->
  <xsl:for-each select="*">
    <xsl:choose>
      <!-- Handle Note record -->
      <xsl:when test="name(current()) = 'NoteRecord'">
        <xsl:apply-templates select="current()"/>
      </xsl:when>

      <!-- Handle Point record -->
      <xsl:when test="name(current()) = 'PointRecord'">
        <xsl:apply-templates select="current()"/>
      </xsl:when>

      <!-- Handle Station record -->
      <xsl:when test="name(current()) = 'StationRecord'">
        <xsl:apply-templates select="current()"/>
      </xsl:when>

      <!-- Handle Reference record -->
      <xsl:when test="name(current()) = 'ReferenceRecord'">
        <xsl:apply-templates select="current()"/>
      </xsl:when>

      <!-- Handle Line record -->
      <xsl:when test="name(current()) = 'LineRecord'">
        <xsl:apply-templates select="current()"/>
      </xsl:when>

      <!-- Process any notes attached to the current record -->
      <xsl:when test="current()/Notes">
        <xsl:apply-templates select="current()/Notes"/>
      </xsl:when>

    </xsl:choose>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** File header Output ************************ -->
<!-- **************************************************************** -->
<xsl:template name="FileHeader">
  <xsl:text> 00 Output from </xsl:text>
  <xsl:value-of select="$product"/>
  <xsl:text> job</xsl:text>
  <xsl:call-template name="NewLine"/>

  <!-- Output the job name in a 01 Administrative block line at the start of the file -->
  <xsl:variable name="JobStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="12"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="JOBFile/@jobName" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <!-- Use file TimeStamp attribute for date -->
  <xsl:variable name="DateStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="12"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="concat(substring(JOBFile/@TimeStamp, 9, 2),
                                     substring(JOBFile/@TimeStamp, 6, 2),
                                     substring(JOBFile/@TimeStamp, 1, 4),
                                     ' 002')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <!-- Use the JobPropertiesRecord/Reference for the Road Number -->
  <!-- The latest header details don't mention a road number at all just 'KKKKKKK KKKK' so -->
  <!-- write out 12 spaces instead of useing the Reference details.                        -->
  <xsl:variable name="RoadNr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="12"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="' '" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <!-- The Municipal field now appears to be fixed as '$111' -->
  <xsl:variable name="Municipal">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="12"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="'$111'" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <!-- Use the JobPropertiesRecord/Operator for the Surveyor -->
  <xsl:variable name="Surveyor">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="12"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="JOBFile/FieldBook/JobPropertiesRecord/Operator" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat(' 01 ', $JobStr, ' ', $DateStr, ' ', $RoadNr, ' ', $Municipal, ' ', $Surveyor)"/>  
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************* NoteRecord Output ************************ -->
<!-- **************************************************************** -->
<xsl:template match="NoteRecord">
  <xsl:apply-templates select="Notes"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** Attached Note Output ********************** -->
<!-- **************************************************************** -->
<xsl:template match="Notes">
  <!-- Process all notes attached to the current record -->
  <xsl:for-each select="current()/Note">
    <xsl:variable name="prefixStr">
      <xsl:choose>
        <xsl:when test="contains(current(), 'VRS base:')">
          <xsl:value-of select="' 00 '"/> <!-- Use 00 prefix for the VRS position -->
        </xsl:when>
        <xsl:when test="starts-with(current(), '11')">  <!-- When the note starts with 11 output with 11 prefix -->
          <xsl:value-of select="' 11 '"/>
        </xsl:when>
        <xsl:when test="starts-with(current(), '12')">  <!-- When the note starts with 12 output with 12 prefix -->
          <xsl:value-of select="' 12 '"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="' 09 '"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- Now form up the note string appropriately -->
    <xsl:variable name="noteStr">
      <xsl:choose>
        <xsl:when test="starts-with(current(), '09')"> <!-- If the note starts with '09' remove it since the prefixStr is already set -->
          <xsl:value-of select="substring(current(), 4)"/>  <!-- Take everything from the 4th character on -->
        </xsl:when>

        <xsl:when test="starts-with(current(), '11')"> <!-- If the note starts with '11' remove it since the prefixStr is already set -->
          <xsl:variable name="temp" select="substring(current(), 4)"/>
          <!-- Now split up into different fields based on the '*' characters -->
          <xsl:variable name="pppp">
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="40"/>
              <xsl:with-param name="theString" select="substring-before($temp, '*')"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="temp1" select="substring-after($temp, '*')"/>
          <xsl:variable name="ssss">
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="6"/>
              <xsl:with-param name="theString" select="substring-before($temp1, '*')"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="temp2" select="substring-after($temp1, '*')"/>
          <xsl:variable name="iiii">
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="4"/>
              <xsl:with-param name="theString" select="substring-before($temp2, '*')"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="temp3" select="substring-after($temp2, '*')"/>
          <xsl:variable name="uuuu">
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="10"/>
              <xsl:with-param name="theString" select="substring-before($temp3, '*')"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="temp4" select="substring-after($temp3, '*')"/>
          <xsl:variable name="dddd">
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="6"/>
              <xsl:with-param name="theString" select="substring-before($temp4, '*')"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="ss">
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="2"/>
              <xsl:with-param name="theString" select="substring-after($temp4, '*')"/>
            </xsl:call-template>
          </xsl:variable>
          <!-- Now return all the individual strings added together separated by spaces -->
          <xsl:value-of select="concat($pppp, ' ', $ssss, ' ', $iiii, ' ', $uuuu, ' ', $dddd, ' ', $ss)"/>
        </xsl:when>

        <xsl:when test="starts-with(current(), '12')"> <!-- If the note starts with '12' remove it since the prefixStr is already set -->
          <xsl:call-template name="PadRight">
            <xsl:with-param name="stringWidth" select="64"/>
            <xsl:with-param name="theString" select="substring(current(), 4)"/> <!-- Take everything from the 4th character on -->
          </xsl:call-template>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="current()"/> <!-- Just take the current note string -->
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:value-of select="$prefixStr"/>
    <xsl:value-of select="$noteStr"/>
    <xsl:call-template name="NewLine"/>
  </xsl:for-each>  
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** PointRecord Output ************************ -->
<!-- **************************************************************** -->
<xsl:template match="PointRecord">
  <xsl:if test="Deleted = 'false'">  <!-- only output if not deleted -->

    <!-- If the point has any attached notes then output them now -->
    <xsl:if test="Notes">
      <xsl:apply-templates select="Notes"/>
    </xsl:if>

    <xsl:if test="Grid and (Classification != 'Averaged')">
      <xsl:call-template name="GridPoint"/>
    </xsl:if>

    <xsl:if test="Circle">
      <xsl:call-template name="Observation"/> 
    </xsl:if>

    <xsl:if test="(Classification != 'BackSight') and (Classification != 'Averaged') and ComputedGrid and
                  (string(number(ComputedGrid/North)) != 'NaN') and (string(number(ComputedGrid/East)) != 'NaN')">
      <xsl:call-template name="GridPoint">
        <xsl:with-param name="computedGrid" select="'true'"/>
      </xsl:call-template>
    </xsl:if>

    <xsl:if test="ECEFDeltas">
      <xsl:call-template name="GPSDeltas"/>
    </xsl:if>

    <xsl:if test="ECEF">
      <xsl:call-template name="VRSPos"/>
    </xsl:if>

    <xsl:if test="Classification = 'Averaged'">
      <xsl:call-template name="AveragedPosnCommentRecord"/>
    </xsl:if>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- **************** Grid Point Details Output ********************* -->
<!-- **************************************************************** -->
<xsl:template name="GridPoint">
  <xsl:param name="computedGrid" select="'false'"/>
  
  <xsl:if test="($computedGrid = 'false') or (($computedGrid != 'false') and (string(number(ComputedGrid/North)) != 'NaN'))">
    <xsl:variable name="PtStr">
      <xsl:call-template name="PadRight">
        <xsl:with-param name="stringWidth" select="10"/>
        <xsl:with-param name="theString">
          <xsl:value-of select="Name" />
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
  
    <xsl:variable name="NthStr">
      <xsl:call-template name="PadLeft">
        <xsl:with-param name="stringWidth" select="12"/>
        <xsl:with-param name="theString">
          <xsl:variable name="North">
            <xsl:choose>
              <xsl:when test="$computedGrid = 'false'">
                <xsl:value-of select="Grid/North"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="ComputedGrid/North"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="string(number($North))='NaN'"><xsl:value-of select="string('            ')"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="format-number($North * $DistConvFactor, $DecPl3, 'Standard')" /></xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
  
    <xsl:variable name="EastStr">
      <xsl:call-template name="PadLeft">
        <xsl:with-param name="stringWidth" select="11"/>
        <xsl:with-param name="theString">
          <xsl:variable name="East">
            <xsl:choose>
              <xsl:when test="$computedGrid = 'false'">
                <xsl:value-of select="Grid/East"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="ComputedGrid/East"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="string(number($East))='NaN'"><xsl:value-of select="string('           ')"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="format-number($East * $DistConvFactor, $DecPl3, 'Standard')" /></xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
  
    <xsl:variable name="ElevStr">
      <xsl:call-template name="PadLeft">
        <xsl:with-param name="stringWidth" select="8"/>
        <xsl:with-param name="theString">
          <xsl:variable name="Elev">
            <xsl:choose>
              <xsl:when test="$computedGrid = 'false'">
                <xsl:value-of select="Grid/Elevation"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="ComputedGrid/Elevation"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="string(number($Elev))='NaN'"><xsl:value-of select="string('        ')"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="format-number($Elev * $DistConvFactor, $DecPl3, 'Standard')"/></xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
  
    <xsl:variable name="CodeStr">
      <xsl:call-template name="PadRight">
        <xsl:with-param name="stringWidth" select="8"/>
        <xsl:with-param name="theString">
          <xsl:value-of select="Code" />
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
  
    <xsl:variable name="Bk" select="'  '"/>         <!-- Calculation code - output as blank (2 chars) -->
    <xsl:variable name="Merk">                      <!-- Note (Merk) - output Description 2 with maximum of 7 chars -->
      <xsl:call-template name="PadRight">
        <xsl:with-param name="stringWidth" select="7"/>
        <xsl:with-param name="theString" select="Description2"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="computationComment">
      <xsl:if test="(Method = 'Resection') or Circle and not(Stakeout)">Beregnet koordinat</xsl:if>
    </xsl:variable>

    <!-- Now output the formatted Coordinate Block -->
    <xsl:choose>
      <xsl:when test="Method = 'Resection'"> 00 </xsl:when>
      <xsl:when test="Circle"> 00 </xsl:when>
      <xsl:otherwise> 05 </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="concat($PtStr, ' ', $CodeStr, ' ', $NthStr, ' ', $EastStr, ' ', $ElevStr, ' ', $Bk, ' ', $computationComment, ' ', $Merk)"/>
    <xsl:call-template name="NewLine"/>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****** Output a comment record with the averaged position ****** -->
<!-- **************************************************************** -->
<xsl:template name="AveragedPosnCommentRecord">

  <xsl:text> 00 </xsl:text>  <!-- Indicates a comment record -->

  <!-- Output point name -->
  <xsl:call-template name="PadRight">
    <xsl:with-param name="stringWidth" select="10"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="Name" />
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>
  
  <!-- Output point code -->
  <xsl:call-template name="PadRight">
    <xsl:with-param name="stringWidth" select="8"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="Code" />
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <!-- Output north value -->
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:variable name="north">
        <xsl:choose>
          <xsl:when test="string(number(Grid/North)) != 'NaN'">
            <xsl:value-of select="Grid/North"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="ComputedGrid/North"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="string(number($north))='NaN'"><xsl:value-of select="string('            ')"/></xsl:when>
        <xsl:otherwise><xsl:value-of select="format-number($north * $DistConvFactor, $DecPl3, 'Standard')" /></xsl:otherwise>
      </xsl:choose>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <!-- Output east value -->
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="11"/>
    <xsl:with-param name="theString">
      <xsl:variable name="east">
        <xsl:choose>
          <xsl:when test="string(number(Grid/East)) != 'NaN'">
            <xsl:value-of select="Grid/East"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="ComputedGrid/East"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="string(number($east))='NaN'"><xsl:value-of select="string('            ')"/></xsl:when>
        <xsl:otherwise><xsl:value-of select="format-number($east * $DistConvFactor, $DecPl3, 'Standard')" /></xsl:otherwise>
      </xsl:choose>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <!-- Output elevation value -->
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="8"/>
    <xsl:with-param name="theString">
      <xsl:variable name="elev">
        <xsl:choose>
          <xsl:when test="string(number(Grid/Elevation)) != 'NaN'">
            <xsl:value-of select="Grid/Elevation"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="ComputedGrid/Elevation"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="string(number($elev))='NaN'"><xsl:value-of select="string('            ')"/></xsl:when>
        <xsl:otherwise><xsl:value-of select="format-number($elev * $DistConvFactor, $DecPl3, 'Standard')" /></xsl:otherwise>
      </xsl:choose>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:choose>
    <xsl:when test="count(AverageResiduals/AverageResidual/AngleResidual/HorizontalCircle[string(number(.)) != 'NaN']) = 0">
      <xsl:value-of select="'  Midlet verdi        '"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="'    Midlet koordinat        '"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- **************** Observation Details Output ******************** -->
<!-- **************************************************************** -->
<xsl:template name="Observation">
  <xsl:variable name="PtStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="10"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="Name" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="HzStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:call-template name="FormatAngle">
          <xsl:with-param name="theAngle" select="Circle/HorizontalCircle"/>
          <xsl:with-param name="gonsDecPlaces">4</xsl:with-param>
          <xsl:with-param name="dmsSymbols">.</xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="VtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:call-template name="FormatAngle">
          <xsl:with-param name="theAngle" select="Circle/VerticalCircle"/>
          <xsl:with-param name="gonsDecPlaces">4</xsl:with-param>
          <xsl:with-param name="dmsSymbols">.</xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="prismConst">
    <xsl:for-each select="key('tgtID-search', TargetID)[1]">
      <xsl:value-of select="PrismConstant"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="SDStr">  <!-- Apply prism constant to the distances -->
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:variable name="SD" select="Circle/EDMDistance" />
        <xsl:choose>
          <xsl:when test="string(number(Circle/EDMDistance))='NaN'"><xsl:value-of select="string('        ')"/></xsl:when>
          <xsl:otherwise><xsl:value-of select="format-number((Circle/EDMDistance + $prismConst) * $DistConvFactor, $DecPl3, 'Standard')"/></xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="CodeStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="Code" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="TgtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="6"/>
      <xsl:with-param name="theString">
        <xsl:for-each select="key('tgtID-search', TargetID)[1]">
          <xsl:value-of select="format-number(TargetHeight * $DistConvFactor, $DecPl3, 'Standard')"/>
        </xsl:for-each>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="Bk" select="'  '"/>         <!-- Calculation code - output as blank (2 chars) -->
  <xsl:variable name="Merk">                      <!-- Note (Merk) - output Description 2 with maximum of 7 chars -->
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="7"/>
      <xsl:with-param name="theString" select="Description2"/>
    </xsl:call-template>
  </xsl:variable>

  <!-- Now output the formatted Measurement Block -->
  <xsl:value-of select="concat(' 03 ', $PtStr, ' ', $CodeStr, ' ', $HzStr, ' ', $VtStr, ' ', $SDStr, ' ', $TgtStr, ' ', $Bk, ' ', $Merk)"/>
  <xsl:call-template name="NewLine"/>

  <!-- If this is a backsight observation then look for any station residuals -->
  <!-- for it and write them out.                                             -->
  <xsl:if test="Classification = 'BackSight'">
    <xsl:variable name="obsID" select="@ID"/>   <!-- Retain the PointRecord ID -->
    <xsl:for-each select="following-sibling::StationResiduals[1]">
      <xsl:for-each select="ResidualsRecord[ObservationID = $obsID]">
        <!-- There is a ResidualsRecord for this observation so output the deltas -->
        <xsl:text> 00 </xsl:text>
        <xsl:text> dN = </xsl:text>
        <xsl:value-of select="format-number(GridResidual/DeltaNorth * $DistConvFactor, $DecPl3, 'Standard')"/>
        <xsl:text>, dE = </xsl:text>
        <xsl:value-of select="format-number(GridResidual/DeltaEast * $DistConvFactor, $DecPl3, 'Standard')"/>
        <xsl:text>, dZ = </xsl:text>
        <xsl:value-of select="format-number(GridResidual/DeltaElevation * $DistConvFactor, $DecPl3, 'Standard')"/>
        <xsl:call-template name="NewLine"/>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:if>

</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** ReferenceRecord Output ********************** -->
<!-- **************************************************************** -->
<xsl:template match="ReferenceRecord">

  <xsl:variable name="ptName" select="PointName"/>
  <xsl:variable name="baseLat">
    <xsl:for-each select="/JOBFile/Reductions/Point[Name = $ptName]">
      <xsl:value-of select="WGS84/Latitude"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="baseLong">
    <xsl:for-each select="/JOBFile/Reductions/Point[Name = $ptName]">
      <xsl:value-of select="WGS84/Longitude"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="baseHeight">
    <xsl:for-each select="/JOBFile/Reductions/Point[Name = $ptName]">
      <xsl:value-of select="WGS84/Height"/>
    </xsl:for-each>
  </xsl:variable>
  
  <xsl:variable name="baseX">
    <xsl:call-template name="ecefX">
      <xsl:with-param name="latitude" select="$baseLat"/>
      <xsl:with-param name="longitude" select="$baseLong"/>
      <xsl:with-param name="height" select="$baseHeight"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="baseY">
    <xsl:call-template name="ecefY">
      <xsl:with-param name="latitude" select="$baseLat"/>
      <xsl:with-param name="longitude" select="$baseLong"/>
      <xsl:with-param name="height" select="$baseHeight"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="baseZ">
    <xsl:call-template name="ecefZ">
      <xsl:with-param name="latitude" select="$baseLat"/>
      <xsl:with-param name="height" select="$baseHeight"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="paddedPtName">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="10"/>
      <xsl:with-param name="theString" select="PointName"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="paddedPtCode">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:for-each select="/JOBFile/Reductions/Point[Name = $ptName]">
          <xsl:value-of select="Code"/>
        </xsl:for-each>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat(' 41 ', $paddedPtName, ' ', $paddedPtCode, '       21                               ')"/>

  <xsl:variable name="antHt">
    <xsl:for-each select="preceding-sibling::AntennaRecord[1]">
      <xsl:value-of select="ReducedHeight"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="paddedAntHt">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="6"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number($antHt * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="Bk" select="'  '"/>         <!-- Calculation code - output as blank (2 chars) -->
  <xsl:variable name="Merk">                      <!-- Note (Merk) - output Description 2 with maximum of 7 chars -->
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="7"/>
      <xsl:with-param name="theString" select="Description2"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat($paddedAntHt, ' ', $Bk, ' ', $Merk)"/>
  <xsl:call-template name="NewLine"/>
  
  <xsl:value-of select="concat(' 42 ', $paddedPtName, ' ', $paddedPtCode, ' ')"/>
  
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number($baseX * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>
  
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number($baseY * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number($baseZ * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:value-of select="concat($paddedAntHt, ' ', $Bk, ' ', $Merk)"/>
  <xsl:call-template name="NewLine"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** GPS Deltas Output *********************** -->
<!-- **************************************************************** -->
<xsl:template name="GPSDeltas">

  <xsl:variable name="antHt">
    <xsl:choose>
      <xsl:when test="not(ComputedGrid) or (string(number(ComputedGrid/North)) = 'NaN') or
                      (string(number(ComputedGrid/North)) = 'NaN')">
        <!-- If there are no valid ComputedGrid values then we need to use the appropriate antenna height -->
        <xsl:for-each select="key('antID-search', AntennaID)[1]">
          <xsl:value-of select="ReducedHeight"/>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>0</xsl:otherwise> <!-- We will output the grid coordinates so set the antenna height to zero -->
    </xsl:choose>
  </xsl:variable>

  <!-- If there are no valid ComputedGrid values output the actual GPS vector details -->
  <xsl:if test="not(ComputedGrid) or (string(number(ComputedGrid/North)) = 'NaN') or
                (string(number(ComputedGrid/North)) = 'NaN')">
    <xsl:call-template name="Record43Output"/>
  </xsl:if>

  <xsl:if test="QualityControl2">  <!-- There is a QualityControl2 element for this point -->
    <xsl:call-template name="Record47Output"/>
  </xsl:if>

  <!-- Output a 46 record with the date, time, Nbr of SVs, PDOP, Antenna Ht, Nbr positions, Observation class -->
  <xsl:call-template name="Record46Output">
    <xsl:with-param name="antHt" select="$antHt"/>
  </xsl:call-template>

  <!-- Output a comment record with the precisions and other details -->
  <xsl:call-template name="GPSDataCommentOutput"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************* VRS Position Output ********************** -->
<!-- **************************************************************** -->
<xsl:template name="VRSPos">

  <xsl:variable name="antHt">
    <xsl:choose>
      <xsl:when test="not(ComputedGrid) or (string(number(ComputedGrid/North)) = 'NaN') or
                      (string(number(ComputedGrid/North)) = 'NaN')">
        <!-- If there are no valid ComputedGrid values then we need to use the appropriate antenna height -->
        <xsl:for-each select="key('antID-search', AntennaID)[1]">
          <xsl:value-of select="ReducedHeight"/>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>0</xsl:otherwise> <!-- We will output the grid coordinates so set the antenna height to zero -->
    </xsl:choose>
  </xsl:variable>

  <!-- If there are no valid ComputedGrid values output the actual GPS vector details -->
  <xsl:if test="not(ComputedGrid) or (string(number(ComputedGrid/North)) = 'NaN') or
                (string(number(ComputedGrid/North)) = 'NaN')">
    <xsl:call-template name="Record45Output"/>
  </xsl:if>

  <!-- Output a 47 record with the covariance values -->
  <xsl:if test="QualityControl2">  <!-- There is a QualityControl2 element for this point -->
    <xsl:call-template name="Record47Output"/>
  </xsl:if>

  <!-- Output a 46 record with the date, time, Nbr of SVs, PDOP, Antenna Ht, Nbr positions, Observation class   -->
  <!-- Since the grid coords (05) record will be output for the point position and this already has the antenna -->
  <!-- applied, the 46 record should output a zero antenna height - this will occur by default if no antenna    -->
  <!-- height parameter is passed to the function.                                                              -->
  <xsl:call-template name="Record46Output">
    <xsl:with-param name="antHt" select="$antHt"/>
  </xsl:call-template>

  <!-- Output a comment record with the precisions and other details -->
  <xsl:call-template name="GPSDataCommentOutput"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- *************** Output a 43 (GPS Vector) Record **************** -->
<!-- **************************************************************** -->
<xsl:template name="Record43Output">
  <xsl:value-of select="' 43 '"/>

  <xsl:call-template name="PadRight">
    <xsl:with-param name="stringWidth" select="10"/>
    <xsl:with-param name="theString" select="Name"/>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadRight">
    <xsl:with-param name="stringWidth" select="8"/>
    <xsl:with-param name="theString" select="Code"/>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(ECEFDeltas/DeltaX  * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(ECEFDeltas/DeltaY  * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(ECEFDeltas/DeltaZ  * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:variable name="antHt">
    <xsl:for-each select="key('antID-search', AntennaID)[1]">
      <xsl:value-of select="ReducedHeight"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="6"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number($antHt * $DistConvFactor, $DecPl3, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:variable name="Bk" select="'  '"/>         <!-- Calculation code - output as blank (2 chars) -->
  <xsl:variable name="Merk">                      <!-- Note (Merk) - output Description 2 with maximum of 7 chars -->
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="7"/>
      <xsl:with-param name="theString" select="Description2"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat(' ', $Bk, ' ', $Merk)"/>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- *************** Output a 44 (Covariance) Record **************** -->
<!-- **************************************************************** -->
<xsl:template name="Record44Output">
  <!-- Output 44 record with covariance values
       44 MMM.xxxx MMM.yyyy MMM.zzzz RR.xxxx RR.xyxy RR.xzxz RR.yyyy RR.yzyz RR.zzzz

       RR.xxxx, RR.yyyy and RR.zzzz is always 1.0000

       RR.xyxy = Vxy/Sqrt(Vxx*Vyy)
       RR.xzxz = Vxz/Sqrt(Vxx*Vzz)
       RR.yzyz = Vyz/Sqrt(Vyy*Vzz)

       MMM.xxxx := Sqrt(Vxx)
       MMM.yyyy := Sqrt(Vyy)
       MMM.zzzz := Sqrt(Vzz);

       Vxx, Vyy, Vzz, Vxy, Vxz, Vyz is the covariance (from QC2-record in the dc-file)
  -->
  <xsl:variable name="MMMxxxx">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="9"/>
      <xsl:with-param name="theString">
        <xsl:variable name="temp">
          <xsl:call-template name="Sqrt">
            <xsl:with-param name="num" select="QualityControl2/VCVxx"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="format-number($temp, $DecPl4, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="MMMyyyy">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="9"/>
      <xsl:with-param name="theString">
        <xsl:variable name="temp">
          <xsl:call-template name="Sqrt">
            <xsl:with-param name="num" select="QualityControl2/VCVyy"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="format-number($temp, $DecPl4, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="MMMzzzz">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="9"/>
      <xsl:with-param name="theString">
        <xsl:variable name="temp">
          <xsl:call-template name="Sqrt">
            <xsl:with-param name="num" select="QualityControl2/VCVzz"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="format-number($temp, $DecPl4, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="sqrtVxxVyy">
    <xsl:call-template name="Sqrt">
      <xsl:with-param name="num" select="QualityControl2/VCVxx * QualityControl2/VCVyy"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="sqrtVxxVzz">
    <xsl:call-template name="Sqrt">
      <xsl:with-param name="num" select="QualityControl2/VCVxx * QualityControl2/VCVzz"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="sqrtVyyVzz">
    <xsl:call-template name="Sqrt">
      <xsl:with-param name="num" select="QualityControl2/VCVyy * QualityControl2/VCVzz"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="RRxxxx" select="'  1.0000'"/>

  <xsl:variable name="RRxyxy">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number(QualityControl2/VCVxy div $sqrtVxxVyy, $DecPl4, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="RRxzxz">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number(QualityControl2/VCVxz div $sqrtVxxVzz, $DecPl4, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="RRyyyy" select="'  1.0000'"/>

  <xsl:variable name="RRyzyz">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="format-number(QualityControl2/VCVyz div $sqrtVyyVzz, $DecPl4, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="RRzzzz" select="'  1.0000'"/>

  <xsl:value-of select="concat(' 44', $MMMxxxx, $MMMyyyy, $MMMzzzz, $RRxxxx, $RRxyxy, $RRxzxz, $RRyyyy, $RRyzyz, $RRzzzz)"/>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********** Output a 45 (GPS Cartesian Coords) Record *********** -->
<!-- **************************************************************** -->
<xsl:template name="Record45Output">
  <xsl:value-of select="' 45 '"/>

  <xsl:call-template name="PadRight">
    <xsl:with-param name="stringWidth" select="10"/>
    <xsl:with-param name="theString" select="Name"/>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadRight">
    <xsl:with-param name="stringWidth" select="8"/>
    <xsl:with-param name="theString" select="Code"/>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(ECEF/X  * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(ECEF/Y  * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(ECEF/Z  * $DistConvFactor, $DecPl4, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="' '"/>

  <xsl:variable name="antHt">
    <xsl:for-each select="key('antID-search', AntennaID)[1]">
      <xsl:value-of select="ReducedHeight"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="6"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number($antHt * $DistConvFactor, $DecPl3, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:variable name="Bk" select="'  '"/>         <!-- Calculation code - output as blank (2 chars) -->
  <xsl:variable name="Merk">                      <!-- Note (Merk) - output Description 2 with maximum of 7 chars -->
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="7"/>
      <xsl:with-param name="theString" select="Description2"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat(' ', $Bk, ' ', $Merk)"/>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Output a 46 (GPS information) Record ************* -->
<!-- **************************************************************** -->
<xsl:template name="Record46Output">
  <xsl:param name="antHt" select="0"/>

  <!-- Output a 46 record with the date, time, Nbr of SVs, PDOP, Antenna Ht, Nbr positions, Observation class -->
  <xsl:value-of select="' 46 '"/>

  <xsl:value-of select="concat(substring(@TimeStamp, 9, 2),
                               substring(@TimeStamp, 6, 2),
                               substring(@TimeStamp, 1, 4), ' ',
                               substring-after(@TimeStamp, 'T'))"/>
  <!-- Output number of satellites -->
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="3"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl1/NumberOfSatellites, $DecPl0, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- Output PDOP -->
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="6"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl1/PDOP, $DecPl2, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- Output Antenna ht -->
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="7"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number($antHt * $DistConvFactor, $DecPl3, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- Output number of positions used -->
  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="4"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl1/NumberOfPositionsUsed, $DecPl0, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- Output measurement type identifier -->
  <xsl:choose>
    <xsl:when test="SurveyMethod = 'Fix'">
      <xsl:value-of select="'  96'"/>
    </xsl:when>
    <xsl:when test="SurveyMethod = 'Float'">
      <xsl:value-of select="'  97'"/>
    </xsl:when>
    <xsl:when test="SurveyMethod = 'NetworkFix'">
      <xsl:value-of select="'  94'"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="'    '"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Output a 47 (Std Covariance) Record ************** -->
<!-- **************************************************************** -->
<xsl:template name="Record47Output">
  <xsl:value-of select="' 47 '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl2/VCVxx, $DecPl8, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl2/VCVyy, $DecPl8, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl2/VCVzz, $DecPl8, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl2/VCVxy, $DecPl8, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl2/VCVxz, $DecPl8, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:value-of select="' '"/>

  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="12"/>
    <xsl:with-param name="theString">
      <xsl:value-of select="format-number(QualityControl2/VCVyz, $DecPl8, 'Standard')"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:value-of select="' '"/>

  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- *************** Output a GPS Data Comment Record *************** -->
<!-- **************************************************************** -->
<xsl:template name="GPSDataCommentOutput">

  <!-- Output a comment record with the precisions and other details -->
  <xsl:value-of select="' 00 '"/>

  <!-- Output measurement type -->
  <xsl:choose>
    <xsl:when test="SurveyMethod = 'Fix'">
      <xsl:value-of select="'Mode:Fine'"/>
    </xsl:when>
    <xsl:when test="SurveyMethod = 'Float'">
      <xsl:value-of select="'Mode:Flyt'"/>
    </xsl:when>
    <xsl:when test="SurveyMethod = 'Code'">
      <xsl:value-of select="'Mode:Flyt'"/>
    </xsl:when>
    <xsl:when test="SurveyMethod = 'WAAS'">
      <xsl:value-of select="'Mode:Flyt'"/>
    </xsl:when>
    <xsl:when test="(SurveyMethod = 'NetworkFix') or (SurveyMethod = 'xFillFix')">
      <xsl:value-of select="'WA Fixed '"/>
    </xsl:when>
    <xsl:when test="(SurveyMethod = 'NetworkFloat') or (SurveyMethod = 'OmniSTARHP') or
                    (SurveyMethod = 'OmniSTARVBS') or (SurveyMethod = 'xFillFloat')">
      <xsl:value-of select="'WA Flyt  '"/>
    </xsl:when>
    <xsl:when test="(SurveyMethod = 'RTK') or (SurveyMethod = 'NetworkRTK') or
                    (SurveyMethod = 'RTKxFill')">
      <!-- Check the preceding InitialisationRecord and output fixed of float -->
      <!-- based on whether or not this indicates initialised or not.         -->
      <xsl:variable name="network">
        <xsl:if test="(SurveyMethod = 'NetworkRTK') or (SurveyMethod = 'RTKxFill')">true</xsl:if>
      </xsl:variable>
      <xsl:for-each select="preceding-sibling::InitialisationRecord[1]">
        <xsl:choose>
          <xsl:when test="InitialisationEvent = 'RTKInitialised'">
            <xsl:choose>
              <xsl:when test="$network = 'true'">WA Fixed </xsl:when>
              <xsl:otherwise>Mode:Fine</xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:choose>
              <xsl:when test="$network = 'true'">WA Flyt  </xsl:when>
              <xsl:otherwise>Mode:Flyt</xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="'         '"/>
    </xsl:otherwise>
  </xsl:choose>

  <xsl:value-of select="'  Prec:'"/>

  <!-- Output Precisions -->
  <xsl:value-of select="concat(format-number(Precision/Horizontal * $DistConvFactor, $DecPl3, 'Standard'), '/',
                               format-number(Precision/Vertical * $DistConvFactor, $DecPl3, 'Standard'))"/>

  <xsl:value-of select="'  PDOP:'"/>

  <!-- Output PDOP -->
  <xsl:value-of select="format-number(QualityControl1/PDOP, $DecPl1, 'Standard')"/>

  <xsl:value-of select="'  Pos:'"/>

  <!-- Output number of positions used -->
  <xsl:value-of select="format-number(QualityControl1/NumberOfPositionsUsed, $DecPl0, 'Standard')"/>

  <xsl:value-of select="'  SVs:'"/>

  <!-- Output number of satellites -->
  <xsl:value-of select="format-number(QualityControl1/NumberOfSatellites, $DecPl0, 'Standard')"/>

  <xsl:value-of select="'  UTC:'"/>

  <!-- Output the date/time - assuming user is recording times as GPS time -->
  <xsl:value-of select="concat(substring(substring-after(@TimeStamp, 'T'), 1, 5), ',',
                               format-number(substring(@TimeStamp, 9, 2), $DecPl0, 'Standard'), '/',
                               format-number(substring(@TimeStamp, 6, 2), $DecPl0, 'Standard'),
                               '-', substring(@TimeStamp, 1, 4))"/>

  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** StationRecord Output ********************** -->
<!-- **************************************************************** -->
<xsl:template match="StationRecord">
  <xsl:variable name="StnStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="10"/>
      <xsl:with-param name="theString">
        <xsl:value-of select="StationName" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="StnCode">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
          <xsl:call-template name="GetStnCode"> <!-- Get station point code from Reductions section -->
            <xsl:with-param name="StnName">
              <xsl:value-of select="StationName"/>
            </xsl:with-param>
          </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="Press">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
          <xsl:call-template name="GetAtmosPress"> <!-- Get pressure from referenced AtmosphereRecord -->
            <xsl:with-param name="AtmosID">
              <xsl:value-of select="AtmosphereID"/>
            </xsl:with-param>
          </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="Temp">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
          <xsl:call-template name="GetAtmosTemp"> <!-- Get temperature from referenced AtmosphereRecord -->
            <xsl:with-param name="AtmosID">
              <xsl:value-of select="AtmosphereID"/>
            </xsl:with-param>
          </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="InstHtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="6"/>
      <xsl:with-param name="theString">
        <xsl:variable name="InstHt" select="TheodoliteHeight" />
        <xsl:value-of select="format-number($InstHt * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="StnTypeStr">   <!-- Setup method (Bk) -->
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="2"/>
      <xsl:with-param name="theString">
        <xsl:if test="StationType[.='StationSetup']">32</xsl:if>
        <xsl:if test="StationType[.='StationSetupPlus']">32</xsl:if>
        <xsl:if test="StationType[.='StandardResection']">31</xsl:if>
        <xsl:if test="StationType[.='HelmertResection']">31</xsl:if>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="DistoIDNbr">  <!-- Presumably the instrument serial number -->
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="stringWidth" select="8"/>
      <xsl:with-param name="theString">
        <xsl:for-each select="key('instID-search', InstrumentID)">
          <xsl:value-of select="Serial"/>
        </xsl:for-each>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="Merk">                      <!-- Note (Merk) - output Description 2 with maximum of 7 chars -->
    <xsl:call-template name="PadRight">
      <xsl:with-param name="stringWidth" select="7"/>
      <xsl:with-param name="theString" select="Description2"/>
    </xsl:call-template>
  </xsl:variable>

  <!-- Now output the formatted Station Block -->
  <xsl:value-of select="concat(' 02 ', $StnStr, ' ', $StnCode, ' ', $DistoIDNbr, ' ', $Press, ' ', $Temp, ' ', $InstHtStr, ' ', $StnTypeStr, ' ', $Merk)"/>
  <xsl:call-template name="NewLine"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- *********************** LineRecord Output ********************** -->
<!-- **************************************************************** -->
<xsl:template match="LineRecord">
  <xsl:if test="Method = 'TwoPoints'">  <!-- Need to output the start and end point coords so need a 'TwoPoints' defined Line -->
    <!-- Get the coordinates of the referenced start point -->
    <xsl:variable name="startNorth">
      <xsl:for-each select="key('reducedPt-search', StartPoint)">
        <xsl:value-of select="Grid/North"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="startEast">
      <xsl:for-each select="key('reducedPt-search', StartPoint)">
        <xsl:value-of select="Grid/East"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="startElev">
      <xsl:for-each select="key('reducedPt-search', StartPoint)">
        <xsl:value-of select="Grid/Elevation"/>
      </xsl:for-each>
    </xsl:variable>

    <!-- Get the coordinates of the referenced end point -->
    <xsl:variable name="endNorth">
      <xsl:for-each select="key('reducedPt-search', EndPoint)">
        <xsl:value-of select="Grid/North"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="endEast">
      <xsl:for-each select="key('reducedPt-search', EndPoint)">
        <xsl:value-of select="Grid/East"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="endElev">
      <xsl:for-each select="key('reducedPt-search', EndPoint)">
        <xsl:value-of select="Grid/Elevation"/>
      </xsl:for-each>
    </xsl:variable>
    
    <xsl:if test="(string(number($startNorth)) != 'NaN') and (string(number($startEast)) != 'NaN') and
                  (string(number($endNorth)) != 'NaN') and (string(number($endEast)) != 'NaN')">
      <!-- We have valid coordinates to use so continue -->
      <xsl:variable name="startNthStr">
        <xsl:call-template name="PadLeft">
          <xsl:with-param name="stringWidth" select="12"/>
          <xsl:with-param name="theString" select="format-number($startNorth * $DistConvFactor, $DecPl3, 'Standard')"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="startEastStr">
        <xsl:call-template name="PadLeft">
          <xsl:with-param name="stringWidth" select="11"/>
          <xsl:with-param name="theString" select="format-number($startEast * $DistConvFactor, $DecPl3, 'Standard')"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="startElevStr">
        <xsl:call-template name="PadLeft">
          <xsl:with-param name="stringWidth" select="8"/>
          <xsl:with-param name="theString">
            <xsl:choose>
              <xsl:when test="string(number($startElev))='NaN'"><xsl:value-of select="string('        ')"/></xsl:when>
              <xsl:otherwise><xsl:value-of select="format-number($startElev * $DistConvFactor, $DecPl3, 'Standard')"/></xsl:otherwise>
            </xsl:choose>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="endNthStr">
        <xsl:call-template name="PadLeft">
          <xsl:with-param name="stringWidth" select="12"/>
          <xsl:with-param name="theString" select="format-number($endNorth * $DistConvFactor, $DecPl3, 'Standard')"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="endEastStr">
        <xsl:call-template name="PadLeft">
          <xsl:with-param name="stringWidth" select="11"/>
          <xsl:with-param name="theString" select="format-number($endEast * $DistConvFactor, $DecPl3, 'Standard')"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="endElevStr">
        <xsl:call-template name="PadLeft">
          <xsl:with-param name="stringWidth" select="8"/>
          <xsl:with-param name="theString">
            <xsl:choose>
              <xsl:when test="string(number($startElev))='NaN'"><xsl:value-of select="string('        ')"/></xsl:when>
              <xsl:otherwise><xsl:value-of select="format-number($endElev * $DistConvFactor, $DecPl3, 'Standard')"/></xsl:otherwise>
            </xsl:choose>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="lineCodeStr">
        <xsl:call-template name="PadRight">
          <xsl:with-param name="stringWidth" select="8"/>
          <xsl:with-param name="theString" select="Code"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="Bk" select="'  '"/>         <!-- Calculation code - output as blank (2 chars) -->
      <xsl:variable name="Merk">                      <!-- Note (Merk) - output Description 2 with maximum of 7 chars -->
        <xsl:call-template name="PadRight">
          <xsl:with-param name="stringWidth" select="7"/>
          <xsl:with-param name="theString" select="Description2"/>
        </xsl:call-template>
      </xsl:variable>

      <!-- Now output the formatted line start point - output a blank point name and use the line code -->
      <xsl:value-of select="concat(' 91 ', '          ', ' ', $lineCodeStr, ' ', $startNthStr, ' ', $startEastStr, ' ', $startElevStr, ' ', $Bk, ' ', $Merk)"/>
      <xsl:call-template name="NewLine"/>

      <!-- Now output the formatted line end point - output a blank point name and use the line code -->
      <xsl:value-of select="concat(' 99 ', '          ', ' ', $lineCodeStr, ' ', $endNthStr, ' ', $endEastStr, ' ', $endElevStr, ' ', $Bk, ' ', $Merk)"/>
      <xsl:call-template name="NewLine"/>
    </xsl:if>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- **** Get Station point code based on stn name in Reductions **** -->
<!-- **************************************************************** -->
<xsl:template name="GetStnCode">
  <xsl:param name="StnName"/>

  <xsl:for-each select="/JOBFile/Reductions/*">
    <xsl:if test="Name=$StnName">
      <xsl:value-of select="Code"/>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Get Atmospheric Pressure based on ID ************* -->
<!-- **************************************************************** -->
<xsl:template name="GetAtmosPress">
  <xsl:param name="AtmosID"/>

  <xsl:for-each select="/JOBFile/FieldBook/*">
    <xsl:if test="@ID=$AtmosID">
      <xsl:variable name="Press" select="Pressure"/>
      <xsl:choose>
        <xsl:when test="string(number($Press))='NaN'"><xsl:value-of select="string(' ')"/></xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="format-number($Press * 0.75006, $DecPl0, 'Standard')"/> <!-- KOF file pressure values always in mmHg -->
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Get Atmospheric Temperature based on ID *********** -->
<!-- **************************************************************** -->
<xsl:template name="GetAtmosTemp">
  <xsl:param name="AtmosID"/>

  <xsl:for-each select="/JOBFile/FieldBook/*">
    <xsl:if test="@ID=$AtmosID">
      <xsl:variable name="Temp" select="Temperature"/>
      <xsl:choose>
        <xsl:when test="string(number($Temp))='NaN'"><xsl:value-of select="string(' ')"/></xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="format-number($Temp, $DecPl0, 'Standard')"/> <!-- KOF file temperature values always in Celsius (as stored in JobXML file) -->
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Output Angle in Appropriate Format **************** -->
<!-- **************************************************************** -->
<xsl:template name="FormatAngle">
  <xsl:param name="theAngle"/>
  <xsl:param name="secDecPlaces" select="0"/>
  <xsl:param name="DMSOutput" select="'false'"/>  <!-- Can be used to force DMS output -->
  <xsl:param name="useSymbols" select="'true'"/>
  <xsl:param name="impliedDecimalPt" select="'false'"/>
  <xsl:param name="gonsDecPlaces" select="5"/>    <!-- Decimal places for gons output -->
  <xsl:param name="decDegDecPlaces" select="5"/>  <!-- Decimal places for decimal degrees output -->
  <xsl:param name="outputAsMilligonsOrSecs" select="'false'"/>
  <xsl:param name="outputAsMilligonsOrSecsSqrd" select="'false'"/>
  <xsl:param name="dmsSymbols">&#0176;'"</xsl:param>

  <xsl:variable name="gonsDecPl">
    <xsl:choose>
      <xsl:when test="$gonsDecPlaces = 1"><xsl:value-of select="$DecPl1"/></xsl:when>
      <xsl:when test="$gonsDecPlaces = 2"><xsl:value-of select="$DecPl2"/></xsl:when>
      <xsl:when test="$gonsDecPlaces = 3"><xsl:value-of select="$DecPl3"/></xsl:when>
      <xsl:when test="$gonsDecPlaces = 4"><xsl:value-of select="$DecPl4"/></xsl:when>
      <xsl:when test="$gonsDecPlaces = 5"><xsl:value-of select="$DecPl5"/></xsl:when>
      <xsl:when test="$gonsDecPlaces = 6"><xsl:value-of select="$DecPl6"/></xsl:when>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="decDegDecPl">
    <xsl:choose>
      <xsl:when test="$decDegDecPlaces = 1"><xsl:value-of select="$DecPl1"/></xsl:when>
      <xsl:when test="$decDegDecPlaces = 2"><xsl:value-of select="$DecPl2"/></xsl:when>
      <xsl:when test="$decDegDecPlaces = 3"><xsl:value-of select="$DecPl3"/></xsl:when>
      <xsl:when test="$decDegDecPlaces = 4"><xsl:value-of select="$DecPl4"/></xsl:when>
      <xsl:when test="$decDegDecPlaces = 5"><xsl:value-of select="$DecPl5"/></xsl:when>
      <xsl:when test="$decDegDecPlaces = 6"><xsl:value-of select="$DecPl6"/></xsl:when>
    </xsl:choose>
  </xsl:variable>

  <xsl:choose>
    <!-- Null angle value -->
    <xsl:when test="string(number($theAngle))='NaN'">
      <xsl:value-of select="format-number($theAngle, $DecPl3, 'Standard')"/> <!-- Use the defined null format output -->
    </xsl:when>
    <!-- There is an angle value -->
    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test="($AngleUnit = 'DMSDegrees') or not($DMSOutput = 'false')">
          <xsl:choose>
            <xsl:when test="$outputAsMilligonsOrSecs != 'false'">
              <xsl:value-of select="format-number($theAngle * $AngleConvFactor * 3600.0, '00.0', 'Standard')"/>
            </xsl:when>            
            <xsl:when test="$outputAsMilligonsOrSecsSqrd != 'false'">
              <xsl:value-of select="format-number($theAngle * $AngleConvFactor * 3600.0 * 3600.0, '00.000', 'Standard')"/>
            </xsl:when>            
            <xsl:otherwise>
              <xsl:call-template name="FormatDMSAngle">
                <xsl:with-param name="decimalAngle" select="$theAngle"/>
                <xsl:with-param name="secDecPlaces" select="$secDecPlaces"/>
                <xsl:with-param name="useSymbols" select="$useSymbols"/>
                <xsl:with-param name="impliedDecimalPt" select="$impliedDecimalPt"/>
                <xsl:with-param name="dmsSymbols" select="$dmsSymbols"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>  
        </xsl:when>

        <xsl:otherwise>
          <xsl:variable name="fmtAngle">
            <xsl:choose>
              <xsl:when test="($AngleUnit = 'Gons') and ($DMSOutput = 'false')">
                <xsl:choose>
                  <xsl:when test="$outputAsMilligonsOrSecs != 'false'">
                    <xsl:value-of select="format-number($theAngle * $AngleConvFactor * 1000.0, $DecPl2, 'Standard')"/>
                  </xsl:when>
                  <xsl:when test="$outputAsMilligonsOrSecsSqrd != 'false'">
                    <xsl:value-of select="format-number($theAngle * $AngleConvFactor * 1000.0 * 1000.0, $DecPl4, 'Standard')"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:choose>
                      <xsl:when test="$secDecPlaces &gt; 0">  <!-- More accurate angle output required -->
                        <xsl:value-of select="format-number($theAngle * $AngleConvFactor, $DecPl8, 'Standard')"/>
                      </xsl:when>
                      <xsl:otherwise>
                        <xsl:value-of select="format-number($theAngle * $AngleConvFactor, $gonsDecPl, 'Standard')"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>

              <xsl:when test="($AngleUnit = 'Mils') and ($DMSOutput = 'false')">
                <xsl:choose>
                  <xsl:when test="$secDecPlaces &gt; 0">  <!-- More accurate angle output required -->
                    <xsl:value-of select="format-number($theAngle * $AngleConvFactor, $DecPl6, 'Standard')"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="format-number($theAngle * $AngleConvFactor, $DecPl4, 'Standard')"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>

              <xsl:when test="($AngleUnit = 'DecimalDegrees') and ($DMSOutput = 'false')">
                <xsl:choose>
                  <xsl:when test="$secDecPlaces &gt; 0">  <!-- More accurate angle output required -->
                    <xsl:value-of select="format-number($theAngle * $AngleConvFactor, $DecPl8, 'Standard')"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="format-number($theAngle * $AngleConvFactor, $decDegDecPl, 'Standard')"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
            </xsl:choose>
          </xsl:variable>
          
          <xsl:choose>
            <xsl:when test="$impliedDecimalPt != 'true'">
              <xsl:value-of select="$fmtAngle"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="translate($fmtAngle, '.', '')"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the right with spaces ************** -->
<!-- **************************************************************** -->
<xsl:template name="PadRight">
  <xsl:param name="stringWidth"/>
  <xsl:param name="theString"/>
  <xsl:choose>
    <xsl:when test="$stringWidth = '0'">
      <xsl:value-of select="normalize-space($theString)"/> <!-- Function return value -->
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="paddedStr" select="concat($theString, '                                                                                          ')"/>
      <xsl:value-of select="substring($paddedStr, 1, $stringWidth)"/> <!-- Function return value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the left with spaces *************** -->
<!-- **************************************************************** -->
<xsl:template name="PadLeft">
  <xsl:param name="stringWidth"/>
  <xsl:param name="theString"/>
  <xsl:choose>
    <xsl:when test="$stringWidth = '0'">
      <xsl:value-of select="normalize-space($theString)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="paddedStr" select="concat('                                                            ', $theString)"/>
      <xsl:value-of select="substring($paddedStr, string-length($paddedStr) - $stringWidth + 1)"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** New Line Output ************************* -->
<!-- **************************************************************** -->
<xsl:template name="NewLine">
  <xsl:text>&#10;</xsl:text>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Return the ECEF X value ********************* -->
<!-- **************************************************************** -->
<xsl:template name="ecefX">
  <xsl:param name="latitude"/>   <!-- latitude value required in decimal degrees -->
  <xsl:param name="longitude"/>  <!-- longitude value required in decimal degrees -->
  <xsl:param name="height"/>

  <!-- Get the latitude in radians -->
  <xsl:variable name="radLat">
    <xsl:call-template name="AngleInRadians">
      <xsl:with-param name="theAngle" select="$latitude"/>
    </xsl:call-template>
  </xsl:variable>

  <!-- Get the longitude in radians -->
  <xsl:variable name="radLong">
    <xsl:call-template name="AngleInRadians">
      <xsl:with-param name="theAngle" select="$longitude"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="N">
    <xsl:call-template name="RadiusOfCurvature">
      <xsl:with-param name="latitude" select="$radLat"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="cosLat">
    <!-- Cosine of latitude = sin(Pi / 2 - latitude) -->
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$Pi div 2.0 - $radLat"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="cosLong">
    <!-- Cosine of longitude = sin(Pi / 2 - longitude) -->
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$Pi div 2.0 - $radLong"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="($N + $height) * $cosLat * $cosLong"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Return the ECEF Y value ********************* -->
<!-- **************************************************************** -->
<xsl:template name="ecefY">
  <xsl:param name="latitude"/>   <!-- latitude value required in decimal degrees -->
  <xsl:param name="longitude"/>  <!-- longitude value required in decimal degrees -->
  <xsl:param name="height"/>

  <!-- Get the latitude in radians -->
  <xsl:variable name="radLat">
    <xsl:call-template name="AngleInRadians">
      <xsl:with-param name="theAngle" select="$latitude"/>
    </xsl:call-template>
  </xsl:variable>

  <!-- Get the longitude in radians -->
  <xsl:variable name="radLong">
    <xsl:call-template name="AngleInRadians">
      <xsl:with-param name="theAngle" select="$longitude"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="N">
    <xsl:call-template name="RadiusOfCurvature">
      <xsl:with-param name="latitude" select="$radLat"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="cosLat">
    <!-- Cosine of latitude = sin(Pi / 2 - latitude) -->
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$Pi div 2.0 - $radLat"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="sinLong">
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$radLong"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="($N + $height) * $cosLat * $sinLong"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Return the ECEF Z value ********************* -->
<!-- **************************************************************** -->
<xsl:template name="ecefZ">
  <xsl:param name="latitude"/>  <!-- latitude value required in decimal degrees -->  
  <xsl:param name="height"/>

  <!-- Get the latitude in radians -->
  <xsl:variable name="radLat">
    <xsl:call-template name="AngleInRadians">
      <xsl:with-param name="theAngle" select="$latitude"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="N">
    <xsl:call-template name="RadiusOfCurvature">
      <xsl:with-param name="latitude" select="$radLat"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="sinLat">
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$radLat"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="($N * (1.0 - $WGS84EccentricitySquared) + $height) * $sinLat"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- *************** Return the square root of a value ************** -->
<!-- **************************************************************** -->
<xsl:template name="Sqrt">
  <xsl:param name="num" select="0"/>       <!-- The number you want to find the square root of -->
  <xsl:param name="try" select="1"/>       <!-- The current 'try'.  This is used internally. -->
  <xsl:param name="iter" select="1"/>      <!-- The current iteration, checked against maxiter to limit loop count - used internally -->
  <xsl:param name="maxiter" select="40"/>  <!-- Set this up to insure against infinite loops - used internally -->

  <!-- This template uses Sir Isaac Newton's method of finding roots -->

  <xsl:choose>
    <xsl:when test="$num &lt; 0"></xsl:when>  <!-- Invalid input - no square root of a negative number so return null -->
    <xsl:when test="$try * $try = $num or $iter &gt; $maxiter">
      <xsl:value-of select="$try"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="Sqrt">
        <xsl:with-param name="num" select="$num"/>
        <xsl:with-param name="try" select="$try - (($try * $try - $num) div (2 * $try))"/>
        <xsl:with-param name="iter" select="$iter + 1"/>
        <xsl:with-param name="maxiter" select="$maxiter"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** Format a DMS Angle ********************** -->
<!-- **************************************************************** -->
<xsl:template name="FormatDMSAngle">
  <xsl:param name="decimalAngle"/>
  <xsl:param name="secDecPlaces" select="0"/>
  <xsl:param name="useSymbols" select="'true'"/>
  <xsl:param name="impliedDecimalPt" select="'false'"/>
  <xsl:param name="dmsSymbols">&#0176;'"</xsl:param>

  <xsl:variable name="degreesSymbol">
    <xsl:choose>
      <xsl:when test="$useSymbols = 'true'"><xsl:value-of select="substring($dmsSymbols, 1, 1)"/></xsl:when>  <!-- Degrees symbol ° -->
      <xsl:otherwise>
        <xsl:if test="$impliedDecimalPt != 'true'">.</xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="minutesSymbol">
    <xsl:choose>
      <xsl:when test="$useSymbols = 'true'"><xsl:value-of select="substring($dmsSymbols, 2, 1)"/></xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="secondsSymbol">
    <xsl:choose>
      <xsl:when test="$useSymbols = 'true'"><xsl:value-of select="substring($dmsSymbols, 3, 1)"/></xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="sign">
    <xsl:if test="$decimalAngle &lt; '0.0'">-1</xsl:if>
    <xsl:if test="$decimalAngle &gt;= '0.0'">1</xsl:if>
  </xsl:variable>

  <xsl:variable name="posDecimalDegrees" select="number($decimalAngle * $sign)"/>

  <xsl:variable name="positiveDecimalDegrees">  <!-- Ensure an angle very close to 360° is treated as 0° -->
    <xsl:choose>
      <xsl:when test="(360.0 - $posDecimalDegrees) &lt; 0.00001">
        <xsl:value-of select="0"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$posDecimalDegrees"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="decPlFmt">
    <xsl:choose>
      <xsl:when test="$secDecPlaces = 0"><xsl:value-of select="''"/></xsl:when>
      <xsl:when test="$secDecPlaces = 1"><xsl:value-of select="'.0'"/></xsl:when>
      <xsl:when test="$secDecPlaces = 2"><xsl:value-of select="'.00'"/></xsl:when>
      <xsl:when test="$secDecPlaces = 3"><xsl:value-of select="'.000'"/></xsl:when>
      <xsl:when test="$secDecPlaces = 4"><xsl:value-of select="'.0000'"/></xsl:when>
      <xsl:when test="$secDecPlaces = 5"><xsl:value-of select="'.00000'"/></xsl:when>
      <xsl:when test="$secDecPlaces = 6"><xsl:value-of select="'.000000'"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="''"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="degrees" select="floor($positiveDecimalDegrees)"/>
  <xsl:variable name="decimalMinutes" select="number(number($positiveDecimalDegrees - $degrees) * 60 )"/>
  <xsl:variable name="minutes" select="floor($decimalMinutes)"/>
  <xsl:variable name="seconds" select="number(number($decimalMinutes - $minutes)*60)"/>

  <xsl:variable name="partiallyNormalisedMinutes">
    <xsl:if test="number(format-number($seconds, concat('00', $decPlFmt))) = 60"><xsl:value-of select="number($minutes + 1)"/></xsl:if>
    <xsl:if test="not(number(format-number($seconds, concat('00', $decPlFmt))) = 60)"><xsl:value-of select="$minutes"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="normalisedSeconds">
    <xsl:if test="number(format-number($seconds, concat('00', $decPlFmt))) = 60"><xsl:value-of select="0"/></xsl:if>
    <xsl:if test="not(number(format-number($seconds, concat('00', $decPlFmt))) = 60)"><xsl:value-of select="$seconds"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="partiallyNormalisedDegrees">
    <xsl:if test="format-number($partiallyNormalisedMinutes, '0') = '60'"><xsl:value-of select="number($degrees + 1)"/></xsl:if>
    <xsl:if test="not(format-number($partiallyNormalisedMinutes, '0') = '60')"><xsl:value-of select="$degrees"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="normalisedDegrees">
    <xsl:if test="format-number($partiallyNormalisedDegrees, '0') = '360'"><xsl:value-of select="0"/></xsl:if>
    <xsl:if test="not(format-number($partiallyNormalisedDegrees, '0') = '360')"><xsl:value-of select="$partiallyNormalisedDegrees"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="normalisedMinutes">
    <xsl:if test="format-number($partiallyNormalisedMinutes, '00') = '60'"><xsl:value-of select="0"/></xsl:if>
    <xsl:if test="not(format-number($partiallyNormalisedMinutes, '00') = '60')"><xsl:value-of select="$partiallyNormalisedMinutes"/></xsl:if>
  </xsl:variable>

  <xsl:if test="$sign = -1">-</xsl:if>
  <xsl:value-of select="format-number($normalisedDegrees, '0')"/>
  <xsl:value-of select="$degreesSymbol"/>
  <xsl:value-of select="format-number($normalisedMinutes, '00')"/>
  <xsl:value-of select="$minutesSymbol"/>
  <xsl:choose>
    <xsl:when test="$useSymbols = 'true'">
      <xsl:value-of select="format-number($normalisedSeconds, concat('00', $decPlFmt))"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="translate(format-number($normalisedSeconds, concat('00', $decPlFmt)), '.', '')"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:value-of select="$secondsSymbol"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Return the Radius of Curvature in Prime Vertical ******* -->
<!-- **************************************************************** -->
<xsl:template name="RadiusOfCurvature">
  <xsl:param name="latitude"/>   <!-- Already converted to radians -->

  <xsl:variable name="sinLat">
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$latitude"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="sinLatSquared">
    <xsl:value-of select="$sinLat * $sinLat"/>
  </xsl:variable>

  <xsl:variable name="tempVal">
    <xsl:call-template name="Sqrt">
      <xsl:with-param name="num" select="1.0 - $WGS84EccentricitySquared * $sinLatSquared"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="$WGS84SemiMajorAxis div $tempVal"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** Angle in Radians ************************ -->
<!-- **************************************************************** -->
<xsl:template name="AngleInRadians">
  <xsl:param name="theAngle"/>
  <xsl:param name="normalise" select="'false'"/>
  <xsl:choose>
    <!-- Null angle value -->
    <xsl:when test="string(number($theAngle)) = 'NaN'">
      <xsl:value-of select="''"/>
    </xsl:when>
    <!-- There is an angle value -->
    <xsl:otherwise>
      <xsl:variable name="radiansAngle">
        <xsl:value-of select="$theAngle * $Pi div 180.0"/>
      </xsl:variable>

      <xsl:variable name="outAngle">
        <xsl:choose>
          <xsl:when test="$normalise = 'false'">
            <xsl:value-of select="$radiansAngle"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="RadianAngleBetweenLimits">
              <xsl:with-param name="anAngle" select="$radiansAngle"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:value-of select="$outAngle"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Return the sine of an angle in radians ************ -->
<!-- **************************************************************** -->
<xsl:template name="Sine">
  <xsl:param name="theAngle"/>
  <xsl:variable name="normalisedAngle">
    <xsl:call-template name="RadianAngleBetweenLimits">
      <xsl:with-param name="anAngle" select="$theAngle"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="theSine">
    <xsl:call-template name="sineIter">
      <xsl:with-param name="pX2" select="$normalisedAngle * $normalisedAngle"/>
      <xsl:with-param name="pRslt" select="$normalisedAngle"/>
      <xsl:with-param name="pElem" select="$normalisedAngle"/>
      <xsl:with-param name="pN" select="1"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="number($theSine)"/>
</xsl:template>

<xsl:template name="sineIter">
  <xsl:param name="pX2"/>
  <xsl:param name="pRslt"/>
  <xsl:param name="pElem"/>
  <xsl:param name="pN"/>
  <xsl:param name="pEps" select="0.00000001"/>
  <xsl:variable name="vnextN" select="$pN+2"/>
  <xsl:variable name="vnewElem"  select="-$pElem*$pX2 div ($vnextN*($vnextN - 1))"/>
  <xsl:variable name="vnewResult" select="$pRslt + $vnewElem"/>
  <xsl:variable name="vdiffResult" select="$vnewResult - $pRslt"/>
  <xsl:choose>
    <xsl:when test="$vdiffResult > $pEps or $vdiffResult &lt; -$pEps">
      <xsl:call-template name="sineIter">
        <xsl:with-param name="pX2" select="$pX2"/>
        <xsl:with-param name="pRslt" select="$vnewResult"/>
        <xsl:with-param name="pElem" select="$vnewElem"/>
        <xsl:with-param name="pN" select="$vnextN"/>
        <xsl:with-param name="pEps" select="$pEps"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$vnewResult"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Return radians angle between Specified Limits ******** -->
<!-- **************************************************************** -->
<xsl:template name="RadianAngleBetweenLimits">
  <xsl:param name="anAngle"/>
  <xsl:param name="minVal" select="0.0"/>
  <xsl:param name="maxVal" select="$Pi * 2.0"/>
  <xsl:param name="incVal" select="$Pi * 2.0"/>

  <xsl:variable name="angle1">
    <xsl:call-template name="AngleValueLessThanMax">
      <xsl:with-param name="inAngle" select="$anAngle"/>
      <xsl:with-param name="maxVal" select="$maxVal"/>
      <xsl:with-param name="incVal" select="$incVal"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="angle2">
    <xsl:call-template name="AngleValueGreaterThanMin">
      <xsl:with-param name="inAngle" select="$angle1"/>
      <xsl:with-param name="minVal" select="$minVal"/>
      <xsl:with-param name="incVal" select="$incVal"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="$angle2"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Return radians angle less than Specificed Maximum ****** -->
<!-- **************************************************************** -->
<xsl:template name="AngleValueLessThanMax">
  <xsl:param name="inAngle"/>
  <xsl:param name="maxVal"/>
  <xsl:param name="incVal"/>

  <xsl:choose>
    <xsl:when test="$inAngle &gt; $maxVal">
      <xsl:variable name="newAngle">
        <xsl:value-of select="$inAngle - $incVal"/>
      </xsl:variable>
      <xsl:call-template name="AngleValueLessThanMax">
        <xsl:with-param name="inAngle" select="$newAngle"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$inAngle"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Return radians angle greater than Zero *********** -->
<!-- **************************************************************** -->
<xsl:template name="AngleValueGreaterThanMin">
  <xsl:param name="inAngle"/>
  <xsl:param name="minVal"/>
  <xsl:param name="incVal"/>

  <xsl:choose>
    <xsl:when test="$inAngle &lt; $minVal">
      <xsl:variable name="newAngle">
        <xsl:value-of select="$inAngle + $incVal"/>
      </xsl:variable>
      <xsl:call-template name="AngleValueGreaterThanMin">
        <xsl:with-param name="inAngle" select="$newAngle"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$inAngle"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


</xsl:stylesheet>
