<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"    
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" >

<!-- (c) 2005, Trimble Navigation Limited. All rights reserved.                                -->
<!-- Permission is hereby granted to use, copy, modify, or distribute this style sheet for any -->
<!-- purpose and without fee, provided that the above copyright notice appears in all copies   -->
<!-- and that both the copyright notice and the limited warranty and restricted rights notice  -->
<!-- below appear in all supporting documentation.                                             -->

<!-- TRIMBLE NAVIGATION LIMITED PROVIDES THIS STYLE SHEET "AS IS" AND WITH ALL FAULTS.         -->
<!-- TRIMBLE NAVIGATION LIMITED SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY -->
<!-- OR FITNESS FOR A PARTICULAR USE. TRIMBLE NAVIGATION LIMITED DOES NOT WARRANT THAT THE     -->
<!-- OPERATION OF THIS STYLE SHEET WILL BE UNINTERRUPTED OR ERROR FREE.                        -->

<xsl:output method="text" omit-xml-declaration="yes" encoding="ISO-8859-1"/>

<!-- Set the numeric display details i.e. decimal point, thousands separator etc -->
<xsl:variable name="DecPt" select="'.'"/>    <!-- Change as appropriate for US/European -->
<xsl:variable name="GroupSep" select="','"/> <!-- Change as appropriate for US/European -->
<!-- Also change decimal-separator & grouping-separator in decimal-format below 
     as appropriate for US/European output -->
<xsl:decimal-format name="Standard" 
                    decimal-separator="."
                    grouping-separator=","
                    infinity="Infinity"
                    minus-sign="-"
                    NaN="?"
                    percent="%"
                    per-mille="&#2030;"
                    zero-digit="0" 
                    digit="#" 
                    pattern-separator=";" />

<xsl:variable name="DecPl0" select="'#0'"/>
<xsl:variable name="DecPl1" select="concat('#0', $DecPt, '0')"/>
<xsl:variable name="DecPl2" select="concat('#0', $DecPt, '00')"/>
<xsl:variable name="DecPl3" select="concat('#0', $DecPt, '000')"/>
<xsl:variable name="DecPl4" select="concat('#0', $DecPt, '0000')"/>
<xsl:variable name="DecPl5" select="concat('#0', $DecPt, '00000')"/>
<xsl:variable name="DecPl8" select="concat('#0', $DecPt, '00000000')"/>

<xsl:variable name="DegreesSymbol" select="'.'"/>
<xsl:variable name="MinutesSymbol" select="''"/>
<xsl:variable name="SecondsSymbol" select="''"/>

<xsl:variable name="fileExt" select="'kof'"/>

<!-- Define a key to speed up search for reduced point details -->
<xsl:key name="reducedPt-search" match="//JOBFile/Reductions/Point" use="ID"/>

<!-- **************************************************************** -->
<!-- Set global variables from the Environment section of JobXML file -->
<!-- **************************************************************** -->
<xsl:variable name="DistUnit"   select="/JOBFile/Environment/DisplaySettings/DistanceUnits" />
<xsl:variable name="AngleUnit"  select="/JOBFile/Environment/DisplaySettings/AngleUnits" />
<xsl:variable name="CoordOrder" select="/JOBFile/Environment/DisplaySettings/CoordinateOrder" />
<xsl:variable name="TempUnit"   select="/JOBFile/Environment/DisplaySettings/TemperatureUnits" />
<xsl:variable name="PressUnit"  select="/JOBFile/Environment/DisplaySettings/PressureUnits" />

<!-- Setup conversion factor for coordinate and distance values -->
<!-- Dist/coord values in JobXML file are always in metres -->
<xsl:variable name="DistConvFactor">
  <xsl:choose>
    <xsl:when test="$DistUnit='Metres'">
      1.0
    </xsl:when>
    <xsl:when test="$DistUnit='InternationalFeet'">
      3.280839895
    </xsl:when>
    <xsl:when test="$DistUnit='USSurveyFeet'">
      3.2808333333357
    </xsl:when>
    <xsl:otherwise>
      1.0
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup conversion factor for angular values -->
<!-- Angular values in JobXML file are always in decimal degrees -->
<xsl:variable name="AngleConvFactor">
  <xsl:choose>
    <xsl:when test="$AngleUnit='DMSDegrees'">
      1.0
    </xsl:when>
    <xsl:when test="$AngleUnit='Gons'">
      1.111111111111
    </xsl:when>
    <xsl:when test="$AngleUnit='Mils'">
      17.77777777777
    </xsl:when>
    <xsl:otherwise>
      1.0
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup boolean variable for coordinate order -->
<xsl:variable name="NECoords">
  <xsl:choose>
    <xsl:when test="$CoordOrder='North-East-Elevation'">
      <xsl:value-of select="'True'"/>
    </xsl:when>
    <xsl:when test="$CoordOrder='X-Y-Z'">
      <xsl:value-of select="'True'"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="'False'"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- Setup conversion factor for pressure values -->
<!-- Pressure values in JobXML file are always in millibars (hPa) -->
<xsl:variable name="PressConvFactor">
  <xsl:choose>
    <xsl:when test="$PressUnit='MilliBar'">
      1.0
    </xsl:when>
    <xsl:when test="$PressUnit='InchHg'">
      0.029529921
    </xsl:when>
    <xsl:when test="$PressUnit='mmHg'">
      0.75006
    </xsl:when>
    <xsl:otherwise>
      1.0
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>


<!-- **************************************************************** -->
<!-- ************************** Main Loop *************************** -->
<!-- **************************************************************** -->
<xsl:template match="/" >

  <!-- Select the FieldBook node to process -->
    <xsl:apply-templates select="JOBFile/FieldBook" />

  <!-- Select Reductions node to process -->
    <!-- Don't use the Reductions node <xsl:apply-templates select="JOBFile/Reductions" /> -->

</xsl:template>


<!-- **************************************************************** -->
<!-- ***************** FieldBook Node Processing ******************** -->
<!-- **************************************************************** -->
<xsl:template match="FieldBook">
<!-- Process the records under the FieldBook node in the order encountered -->
  <xsl:for-each select="*">
    <xsl:choose>
      <!-- Handle Note record -->
      <xsl:when test="name(current()) = 'NoteRecord'">
        <xsl:apply-templates select="current()"/>
      </xsl:when>

      <!-- Handle Corrections record -->
      <xsl:when test="name(current()) = 'CorrectionsRecord'">
        <!-- Skip this record <xsl:apply-templates select="current()"/> -->
      </xsl:when>

      <!-- Handle Point record -->
      <xsl:when test="name(current()) = 'PointRecord'">
        <xsl:apply-templates select="current()"/>
      </xsl:when>

      <!-- Handle Station record -->
      <xsl:when test="name(current()) = 'StationRecord'">
        <!-- Skip this record <xsl:apply-templates select="current()"/> -->
      </xsl:when>

      <!-- Handle BackBearing record -->
      <xsl:when test="name(current()) = 'BackBearingRecord'">
        <!-- Skip this record <xsl:apply-templates select="current()"/> -->
      </xsl:when>

      <!-- Handle Atmosphere record -->
      <xsl:when test="name(current()) = 'AtmosphereRecord'">
        <!-- Skip this record <xsl:apply-templates select="current()"/> -->
      </xsl:when>

      <!-- Handle StationResiduals record -->
      <xsl:when test="name(current()) = 'StationResiduals'">
        <!-- Skip this record <xsl:apply-templates select="current()"/> -->
      </xsl:when>

      <!-- Process any notes attached to the current record -->
      <xsl:when test="current()/Notes">
        <xsl:apply-templates select="current()/Notes"/>
      </xsl:when>

    </xsl:choose>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ***************** Reductions Node Processing ******************* -->
<!-- **************************************************************** -->
<xsl:template match="Reductions">
  <xsl:apply-templates select="Point"/> 
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************* NoteRecord Output ************************ -->
<!-- **************************************************************** -->
<xsl:template match="NoteRecord">
  <xsl:apply-templates select="Notes"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** Attached Note Output ********************** -->
<!-- **************************************************************** -->
<xsl:template match="Notes">
  <!-- Process all notes attached to the current record -->
  <xsl:for-each select="current()/Note">
    <xsl:value-of select="' 09 '"/>
    <xsl:value-of select="current()"/>
    <xsl:call-template name="NewLine"/>
  </xsl:for-each>  
</xsl:template>

<!-- **************************************************************** -->
<!-- ***************** CorrectionsRecord Output ********************* -->
<!-- **************************************************************** -->
<xsl:template match="CorrectionsRecord">
  <xsl:value-of select="'Current Correction Settings:'"/>
  <xsl:call-template name="NewLine"/>

  <xsl:value-of select="'Sea Level Correction       '"/>
    <xsl:if test="SeaLevel[.='true']">On</xsl:if>
    <xsl:if test="SeaLevel[.='false']">Off</xsl:if>
  <xsl:call-template name="NewLine"/>

  <xsl:value-of select="'Curvature Correction       '"/>
    <xsl:if test="Curvature[.='true']">On</xsl:if>
    <xsl:if test="Curvature[.='false']">Off</xsl:if>
  <xsl:call-template name="NewLine"/>

  <xsl:value-of select="'South Azimuth              '"/>
    <xsl:if test="SouthAzimuth[.='true']">Yes</xsl:if>
    <xsl:if test="SouthAzimuth[.='false']">No</xsl:if>
  <xsl:call-template name="NewLine"/>

  <xsl:value-of select="'Grid Orientation           '"/>
    <xsl:if test="GridOrientation[.='IncreasingNorthEast']">Increasing NE</xsl:if>
    <xsl:if test="GridOrientation[.='IncreasingSouthWest']">Increasing SW</xsl:if>
    <xsl:if test="GridOrientation[.='IncreasingNorthWest']">Increasing NW</xsl:if>
    <xsl:if test="GridOrientation[.='IncreasingSouthEast']">Increasing SE</xsl:if>
  <xsl:call-template name="NewLine"/>

  <xsl:value-of select="'Distances as               '"/>
    <xsl:if test="Distances[.='GroundDistance']">Ground</xsl:if>
    <xsl:if test="Distances[.='GridDistance']">Grid</xsl:if>
  <xsl:call-template name="NewLine"/>

  <xsl:value-of select="'Neighbourhood Adjustment   '"/>
    <xsl:if test="NeighbourhoodAdjustment/Applied[.='true']">On</xsl:if>
    <xsl:if test="NeighbourhoodAdjustment/Applied[.='false']">Off</xsl:if>
  <xsl:call-template name="NewLine"/>

  <xsl:value-of select="'Weight Exponent            '"/>
    <xsl:variable name="WeightExp" select="NeighbourhoodAdjustment/WeightExponent" />
    <xsl:value-of select="format-number($WeightExp, $DecPl1, 'Standard')" />    
  <xsl:call-template name="NewLine"/>

  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** PointRecord Output ************************ -->
<!-- **************************************************************** -->
<xsl:template match="PointRecord">
  <xsl:if test="Deleted = 'false'">  <!-- only output if not deleted -->

    <!-- Locate the reduced coordinates (if any) associated with this observation        -->
    <!-- If the point is located in the Reductions section it will be output to the file -->
    <xsl:for-each select="key('reducedPt-search', @ID)">
      <xsl:call-template name="GridPoint"/>
    </xsl:for-each>

    <!-- If the point has any attached notes then output them now -->
    <xsl:if test="Notes">
      <xsl:apply-templates select="Notes"/>
    </xsl:if>

  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- **************** Grid Point Details Output ********************* -->
<!-- **************************************************************** -->
<xsl:template name="GridPoint">
  <xsl:variable name="PtStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="StringWidth" select="10"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="Name" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="NthStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="11"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="North" select="Grid/North" />
        <xsl:choose>
          <xsl:when test="string(number($North))='NaN'"><xsl:value-of select="'           '"/></xsl:when> 
          <xsl:otherwise><xsl:value-of select="format-number($North * $DistConvFactor, $DecPl3, 'Standard')" /></xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="EastStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="12"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="East" select="Grid/East" />
        <xsl:choose>
          <xsl:when test="string(number($East))='NaN'"><xsl:value-of select="'            '"/></xsl:when> 
          <xsl:otherwise><xsl:value-of select="format-number($East * $DistConvFactor, $DecPl3, 'Standard')" /></xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="ElevStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="8"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="Elev" select="Grid/Elevation" />
        <xsl:choose>
          <xsl:when test="string(number($Elev))='NaN'"><xsl:value-of select="'        '"/></xsl:when> 
          <xsl:otherwise><xsl:value-of select="format-number($Elev * $DistConvFactor, $DecPl3, 'Standard')"/></xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="CodeStr">
    <xsl:call-template name="PadRight">
      <xsl:with-param name="StringWidth" select="8"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="Code" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat(' 05 ', $PtStr, ' ', $CodeStr, ' ', $NthStr, ' ', $EastStr, ' ', $ElevStr, '           ')"/>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- **************** Observation Details Output ******************** -->
<!-- **************************************************************** -->
<xsl:template name="Observation">
  <xsl:variable name="NameStr">
    <xsl:value-of select="Name" />
  </xsl:variable>

  <xsl:variable name="BSStr">
    <xsl:if test="Classification = 'BackSight'"> (BS)</xsl:if>
  </xsl:variable>

  <xsl:variable name="PtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="$NameStr" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="HzStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:call-template name="AngleValue">
          <xsl:with-param name="TheAngle">
            <xsl:value-of select="Circle/HorizontalCircle"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="VtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:call-template name="AngleValue">
          <xsl:with-param name="TheAngle">
            <xsl:value-of select="Circle/VerticalCircle"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="SDStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="SD" select="Circle/EDMDistance" />
        <xsl:value-of select="format-number($SD * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="CodeStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="Code" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="TgtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="TH" select="TargetHeight" />
        <xsl:value-of select="format-number($TH * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>


  <!-- Add a second line containing the observation std error values -->

  <xsl:variable name="HzSEStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:call-template name="AngleValue">
          <xsl:with-param name="TheAngle">
            <xsl:value-of select="Circle/HorizontalCircleStandardError"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="VtSEStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:call-template name="AngleValue">
          <xsl:with-param name="TheAngle">
            <xsl:value-of select="Circle/VerticalCircleStandardError"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="SDSEStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="SDErr" select="Circle/EDMDistanceStandardError" />
        <xsl:value-of select="format-number($SDErr * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="PCStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <!-- Locate the target details associated with this observation -->
        <xsl:call-template name="GetTgtDetails">
          <xsl:with-param name="TgtID">
            <xsl:value-of select="TargetID"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="StnStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <!-- Locate the station name associated with this observation -->
        <xsl:call-template name="GetStnName">
          <xsl:with-param name="StnID">
            <xsl:value-of select="StationID"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="BSPtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <!-- Locate the station name associated with this observation -->
        <xsl:call-template name="GetBSPtName">
          <xsl:with-param name="BSID">
            <xsl:value-of select="BackBearingID"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

<!--  <xsl:value-of select="concat('Std Errors            Hz', $HzSEStr, ' Vt', $VtSEStr, ' SD', $SDSEStr, ' Prism C', $PCStr, ' Stn', $StnStr)"/> -->
<!--  <xsl:call-template name="NewLine"/> -->

  <xsl:value-of select="concat($StnStr, ',', $BSPtStr, ',', $PtStr, ',', $HzStr, ',', $VtStr, ',', $SDStr, ',', $CodeStr, ',', $TgtStr)"/>
  <xsl:call-template name="NewLine"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** GPS Deltas Output *********************** -->
<!-- **************************************************************** -->
<xsl:template name="GPSDeltas">
  <xsl:variable name="PtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="16"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="Name" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dXStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="9"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="dX" select="ECEFDeltas/XDelta" />
        <xsl:value-of select="format-number($dX * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dYStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="9"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="dY" select="ECEFDeltas/YDelta" />
        <xsl:value-of select="format-number($dY * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dZStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="9"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="dZ" select="ECEFDeltas/ZDelta" />
        <xsl:value-of select="format-number($dZ * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="CodeStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="20"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="Code" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="BaseStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="16"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="RTK_Base" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat('Point', $PtStr, ' dX', $dXStr, ' dY ', $dYStr, ' dZ', $dZStr, ' Code', $CodeStr, ' Base', $BaseStr)"/>
  <xsl:call-template name="NewLine"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** AtmosphereRecord Output ********************* -->
<!-- **************************************************************** -->
<xsl:template match="AtmosphereRecord">
  <xsl:call-template name="NewLine"/>
  <xsl:value-of select="'Atmospheric Settings:'"/>
  <xsl:call-template name="NewLine"/>

  <xsl:variable name="PressStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="10"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="Press" select="Pressure" />
        <xsl:choose>
          <xsl:when test="string(number($Press))='NaN'"><xsl:value-of select="'?'"/></xsl:when> <!-- could use &#160; for space -->
          <xsl:otherwise>
            <xsl:variable name="PUnit">
              <xsl:choose>
                <xsl:when test="$PressUnit='mmHg'">mmHg</xsl:when>
                <xsl:when test="$PressUnit='InchHg'">inHg</xsl:when>
                <xsl:otherwise>hPa</xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
          <xsl:value-of select="concat(format-number($Press * $PressConvFactor, $DecPl2, 'Standard'), $PUnit)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>
  <xsl:value-of select="concat('Pressure         ', $PressStr)"/>
  <xsl:call-template name="NewLine"/>

  <xsl:variable name="TempStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="10"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="Temp" select="Temperature" />
        <xsl:choose>
          <xsl:when test="string(number($Temp))='NaN'"><xsl:value-of select="'?'"/></xsl:when> <!-- could use &#160; for space -->
          <xsl:otherwise>
            <xsl:variable name="TUnit">
              <xsl:choose>
                <xsl:when test="$TempUnit='Fahrenheit'">&#0176;F</xsl:when>
                <xsl:otherwise>&#0176;C</xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="$TempUnit='Fahrenheit'">
                <xsl:value-of select="concat(format-number($Temp * 1.8 + 32, $DecPl1, 'Standard'), $TUnit)"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="concat(format-number($Temp, $DecPl1, 'Standard'), $TUnit)"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>
  <xsl:value-of select="concat('Temperature      ', $TempStr)"/>
  <xsl:call-template name="NewLine"/>

  <xsl:variable name="ppmStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="10"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="ppm" select="PPM" />
        <xsl:choose>
          <xsl:when test="string(number($ppm))='NaN'"><xsl:value-of select="'?'"/></xsl:when> <!-- could use &#160; for space -->
          <xsl:otherwise><xsl:value-of select="format-number($ppm, $DecPl0, 'Standard')"/></xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>
  <xsl:value-of select="concat('ppm              ', $ppmStr)"/>
  <xsl:call-template name="NewLine"/>

  <xsl:variable name="RefracStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="10"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="Refrac" select="Refraction" />
        <xsl:value-of select="format-number($Refrac, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>
  <xsl:value-of select="concat('Refraction Const.', $RefracStr)"/>
  <xsl:call-template name="NewLine"/>

  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** StationRecord Output ********************** -->
<!-- **************************************************************** -->
<xsl:template match="StationRecord">
  <xsl:variable name="StnStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="StationName" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="InstHtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="InstHt" select="TheodoliteHeight" />
        <xsl:value-of select="format-number($InstHt * $DistConvFactor, $DecPl3, 'Standard')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="StnTypeStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:if test="StationType[.='StationSetup']">Station Setup</xsl:if>
        <xsl:if test="StationType[.='StationSetupPlus']">Station Setup Plus</xsl:if>
        <xsl:if test="StationType[.='StandardResection']">Standard Resection</xsl:if>
        <xsl:if test="StationType[.='HelmertResection']">Helmert Resection</xsl:if>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="SFStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="ScaleFact" select="ScaleFactor" />
        <xsl:variable name="ScaleType" select="ScaleType" />
        <xsl:value-of select="concat(format-number($ScaleFact, $DecPl8, 'Standard'), '(', $ScaleType, ')')" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="SFSEStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="SFStdError" select="ScaleFactorStandardError" />
        <xsl:value-of select="format-number($SFStdError, $DecPl8, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat('Inst Stn,', $StnStr, ',', $InstHtStr, ',', $StnTypeStr, ',', $SFStr, ',', $SFSEStr)"/>
  <xsl:call-template name="NewLine"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- ***************** BackBearingRecord Output ********************* -->
<!-- **************************************************************** -->
<xsl:template match="BackBearingRecord">
  <xsl:variable name="StnStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="Station" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="BSStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="BackSight" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="OrientStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="0"/>
      <xsl:with-param name="TheString">
        <xsl:call-template name="AngleValue">
          <xsl:with-param name="TheAngle">
            <xsl:value-of select="OrientationCorrection"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat('BS,', $StnStr, ',', $BSStr, ',', $OrientStr)"/>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** Format a DMS Angle ********************** -->
<!-- **************************************************************** -->
<xsl:template name="FormatDMSAngle">
  <xsl:param name="DecimalAngle"/>

  <xsl:variable name="Sign">
    <xsl:if test="$DecimalAngle &lt; '0.0'">-1</xsl:if>
    <xsl:if test="$DecimalAngle &gt;= '0.0'">1</xsl:if>
  </xsl:variable>

  <xsl:variable name="PosDecimalDegrees" select="number($DecimalAngle * $Sign)"/>

  <xsl:variable name="PositiveDecimalDegrees">  <!-- Ensure an angle very close to 360° is treated as 0° -->
    <xsl:choose>
      <xsl:when test="(360.0 - $PosDecimalDegrees) &lt; 0.00001">
        <xsl:value-of select="0"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$PosDecimalDegrees"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="Degrees" select="floor($PositiveDecimalDegrees)"/>
  <xsl:variable name="DecimalMinutes" select="number(number($PositiveDecimalDegrees - $Degrees) * 60 )"/>
  <xsl:variable name="Minutes" select="floor($DecimalMinutes)"/>
  <xsl:variable name="Seconds" select="number(number($DecimalMinutes - $Minutes)*60)"/>

  <xsl:variable name="PartialyNormalisedMinutes">
    <xsl:if test="format-number($Seconds, '00') = '60'"><xsl:value-of select="number($Minutes + 1)"/></xsl:if>
    <xsl:if test="not(format-number($Seconds, '00') = '60')"><xsl:value-of select="$Minutes"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="NormalisedSeconds">
    <xsl:if test="format-number($Seconds, '00') = '60'"><xsl:value-of select="0"/></xsl:if>
    <xsl:if test="not(format-number($Seconds, '00') = '60')"><xsl:value-of select="$Seconds"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="PartialyNormalisedDegrees">
    <xsl:if test="format-number($PartialyNormalisedMinutes, '00') = '60'"><xsl:value-of select="number($Degrees + 1)"/></xsl:if>
    <xsl:if test="not(format-number($PartialyNormalisedMinutes, '00') = '60')"><xsl:value-of select="$Degrees"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="NormalisedDegrees">
    <xsl:if test="format-number($PartialyNormalisedDegrees, '00') = '360'"><xsl:value-of select="0"/></xsl:if>
    <xsl:if test="not(format-number($PartialyNormalisedDegrees, '00') = '360')"><xsl:value-of select="$PartialyNormalisedDegrees"/></xsl:if>
  </xsl:variable>

  <xsl:variable name="NormalisedMinutes">
    <xsl:if test="format-number($PartialyNormalisedMinutes, '00') = '60'"><xsl:value-of select="0"/></xsl:if>
    <xsl:if test="not(format-number($PartialyNormalisedMinutes, '00') = '60')"><xsl:value-of select="$PartialyNormalisedMinutes"/></xsl:if>
  </xsl:variable>

  <xsl:value-of select="format-number(number($NormalisedDegrees * $Sign), '00')"/>
  <xsl:value-of select="$DegreesSymbol"/>
  <xsl:value-of select="format-number($NormalisedMinutes, '00')"/>
  <xsl:value-of select="$MinutesSymbol"/>
  <xsl:value-of select="format-number($NormalisedSeconds, '00')"/>
  <xsl:value-of select="$SecondsSymbol"/>
</xsl:template>

<!-- **************************************************************** -->
<!-- ******************* Format a Quadrant Bearing ****************** -->
<!-- **************************************************************** -->
<xsl:template name="FormatQuadrantAngle">
  <xsl:param name="DecimalAngle"/>

  <xsl:variable name="QuadrantAngle">
    <xsl:if test="($DecimalAngle &lt;= '90.0')">
      <xsl:value-of select="number ( $DecimalAngle )"/>
    </xsl:if>
    <xsl:if test="($DecimalAngle &gt; '90.0') and ($DecimalAngle &lt;= '180.0')">
      <xsl:value-of select="number( 180.0 - $DecimalAngle )"/>
    </xsl:if>
    <xsl:if test="($DecimalAngle &gt; '180.0') and ($DecimalAngle &lt; '270.0')">
      <xsl:value-of select="number( $DecimalAngle - 180.0 )"/>
    </xsl:if>
    <xsl:if test="($DecimalAngle &gt;= '270.0') and ($DecimalAngle &lt;= '360.0')">
      <xsl:value-of select="number( 360.0 - $DecimalAngle )"/>
    </xsl:if>
  </xsl:variable>

  <xsl:variable name="QuadrantPrefix">
    <xsl:if test="($DecimalAngle &lt;= '90.0') or ($DecimalAngle &gt;= '270.0')">
       <xsl:text>N</xsl:text>
    </xsl:if>
    <xsl:if test="($DecimalAngle &gt; '90.0') and ($DecimalAngle &lt; '270.0')">
      <xsl:text>S</xsl:text>
    </xsl:if>
  </xsl:variable>

  <xsl:variable name="QuadrantSuffix">
    <xsl:if test="($DecimalAngle &lt;= '180.0')">
      <xsl:text>E</xsl:text>
    </xsl:if>
    <xsl:if test="($DecimalAngle &gt; '180.0')">
      <xsl:text>W</xsl:text>
    </xsl:if>
  </xsl:variable>

  <xsl:value-of select="$QuadrantPrefix"/>
  <xsl:call-template name="FormatDMSAngle">
    <xsl:with-param name="DecimalAngle" select="$QuadrantAngle"/>
  </xsl:call-template>
  <xsl:value-of select="$QuadrantSuffix"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Output Angle in Appropriate Format **************** -->
<!-- **************************************************************** -->
<xsl:template name="AngleValue">
  <xsl:param name="TheAngle"/>
  <xsl:param name="deltaAngle" select="'False'"/>
  <xsl:choose>
    <!-- Null angle value -->
    <xsl:when test="string(number($TheAngle))='NaN'">
      <xsl:value-of select="'?'"/>
    </xsl:when>
    <!-- There is an angle value -->
    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test="$AngleUnit='DMSDegrees'">
          <xsl:call-template name="FormatDMSAngle">
            <xsl:with-param name="DecimalAngle" select="$TheAngle"/>
          </xsl:call-template>
        </xsl:when>

        <xsl:when test="$AngleUnit='Gons'">
          <xsl:value-of select="format-number($TheAngle * $AngleConvFactor, $DecPl5, 'Standard')"/>
        </xsl:when>

        <xsl:when test="$AngleUnit='Mils'">
          <xsl:value-of select="format-number($TheAngle * $AngleConvFactor, $DecPl4, 'Standard')"/>
        </xsl:when>

        <xsl:when test="$AngleUnit='DecimalDegrees'">
          <xsl:value-of select="format-number($TheAngle * $AngleConvFactor, $DecPl5, 'Standard')"/>
        </xsl:when>

        <xsl:when test="$AngleUnit='QuadrantBearings'">
          <xsl:call-template name="FormatQuadrantAngle">
            <xsl:with-param name="DecimalAngle" select="$TheAngle"/>
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Get appropriate target details based on ID *********** -->
<!-- **************************************************************** -->
<xsl:template name="GetTgtDetails">
  <xsl:param name="TgtID"/>

  <xsl:for-each select="//JOBFile/FieldBook/*">
    <xsl:if test="@ID=$TgtID">
      <xsl:variable name="PC" select="PrismConstant"/>
      <xsl:value-of select="concat(format-number($PC *1000.0, $DecPl1, 'Standard'), 'mm')"/>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ***************** Get Station name based on ID ***************** -->
<!-- **************************************************************** -->
<xsl:template name="GetStnName">
  <xsl:param name="StnID"/>

  <xsl:for-each select="//JOBFile/FieldBook/*">
    <xsl:if test="@ID=$StnID">
      <xsl:value-of select="StationName"/>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Get Backsight Station name based on ID ************ -->
<!-- **************************************************************** -->
<xsl:template name="GetBSPtName">
  <xsl:param name="BSID"/>

  <xsl:for-each select="//JOBFile/FieldBook/*">
    <xsl:if test="@ID=$BSID">
      <xsl:value-of select="BackSight"/>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Station Residuals Output ******************** -->
<!-- **************************************************************** -->
<xsl:template match="StationResiduals">
  <xsl:call-template name="NewLine"/>
  <xsl:value-of select="'Station Residuals:'"/>
  <xsl:call-template name="NewLine"/>
  <xsl:apply-templates select="ResidualsRecord"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Residual Details Output ********************* -->
<!-- **************************************************************** -->
<xsl:template match="ResidualsRecord">
  <!-- First line contains the grid residuals -->

  <xsl:variable name="PtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="16"/>
      <xsl:with-param name="TheString">
        <xsl:value-of select="PointName" />
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dNthStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="14"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="dN" select="GridResidual/deltaNorth" />
        <xsl:value-of select="format-number($dN * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dEastStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="14"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="dE" select="GridResidual/deltaEast" />
        <xsl:value-of select="format-number($dE * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dElevStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="14"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="dEl" select="GridResidual/deltaElevation" />
        <xsl:value-of select="format-number($dEl * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$NECoords='True'">
      <xsl:value-of select="concat('Point', $PtStr, ' dN ', $dNthStr, ' dE ', $dEastStr, ' dEl', $dElevStr)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="concat('Point', $PtStr, ' dE ', $dEastStr, ' dN ', $dNthStr, ' dEl', $dElevStr)"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:call-template name="NewLine"/>

  <!-- Second line has the angular and distance residuals -->

  <xsl:variable name="dHzStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="14"/>
      <xsl:with-param name="TheString">
        <xsl:call-template name="AngleValue">
          <xsl:with-param name="TheAngle">
            <xsl:value-of select="AngleResidual/HorizontalCircle"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dVtStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="14"/>
      <xsl:with-param name="TheString">
        <xsl:call-template name="AngleValue">
          <xsl:with-param name="TheAngle">
            <xsl:value-of select="AngleResidual/VerticalCircle"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="dSDStr">
    <xsl:call-template name="PadLeft">
      <xsl:with-param name="StringWidth" select="14"/>
      <xsl:with-param name="TheString">
        <xsl:variable name="dSD" select="AngleResidual/SlopeDistance" />
        <xsl:value-of select="format-number($dSD * $DistConvFactor, $DecPl3, 'Standard')"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="concat('                      dHz', $dHzStr, ' dVt', $dVtStr, ' dSD', $dSDStr)"/>
  <xsl:call-template name="NewLine"/>

  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- *************** Feature and Attributes Output ****************** -->
<!-- **************************************************************** -->
<xsl:template name="OutputAttributes">
  <xsl:param name="PtID"/>
  <xsl:for-each select="//JOBFile/FieldBook/*">
    <xsl:if test="@ID=$PtID">
      <xsl:if test="FeaturesRecord">
        <xsl:for-each select="FeaturesRecord/Feature">
          <xsl:value-of select="'Feature: '"/>
          <xsl:value-of select="@Name"/>
          <xsl:value-of select="' '"/>
          <xsl:for-each select="Attribute">
            <xsl:if test="position() &gt; 1"><xsl:value-of select="','"/></xsl:if>
            <xsl:value-of select="Name"/>
            <xsl:value-of select="','"/>
            <xsl:value-of select="Value"/>
          </xsl:for-each>
        <xsl:call-template name="NewLine"/>
        </xsl:for-each>
      </xsl:if>
    </xsl:if>
  </xsl:for-each>

</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Reductions Point Output ********************* -->
<!-- **************************************************************** -->
<xsl:template match="Point">
  <xsl:call-template name="GridPoint"/> 

  <!-- Now output any Feature and Attribute details associated with
       this point - need to be loacted from the FieldBook section
       associated with the ID of this point -->

  <!-- Locate the FieldBook data associated with this point -->
  <!-- Skip for KOF output
  <xsl:call-template name="OutputAttributes">
    <xsl:with-param name="PtID">
      <xsl:value-of select="ID"/>
    </xsl:with-param>
  </xsl:call-template>
  -->

</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Separating Line Output ********************** -->
<!-- **************************************************************** -->
<xsl:template name="SeparatingLine">
  <hr/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************* Blank Line Output ************************ -->
<!-- **************************************************************** -->
<xsl:template name="BlankLine">
  <xsl:call-template name="NewLine"/>
  <xsl:call-template name="NewLine"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** New Line Output ************************* -->
<!-- **************************************************************** -->
<xsl:template name="NewLine">
<xsl:text>&#xa;</xsl:text>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the left with spaces *************** -->
<!-- **************************************************************** -->
<xsl:template name="PadLeft">
  <xsl:param name="StringWidth"/>
  <xsl:param name="TheString"/>
  <xsl:choose>
    <xsl:when test="$StringWidth = '0'">
      <xsl:value-of select="normalize-space($TheString)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="PaddedStr" select="concat('                                       ', $TheString)"/>
      <xsl:value-of select="substring($PaddedStr, string-length($PaddedStr) - $StringWidth + 1)"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the right with spaces ************** -->
<!-- **************************************************************** -->
<xsl:template name="PadRight">
  <xsl:param name="StringWidth"/>
  <xsl:param name="TheString"/>
  <xsl:choose>
    <xsl:when test="$StringWidth = '0'">
      <xsl:value-of select="normalize-space($TheString)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="PaddedStr" select="concat($TheString, '                                       ')"/>
      <xsl:value-of select="substring($PaddedStr, 1, $StringWidth)"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Return the absolute value ******************* -->
<!-- **************************************************************** -->
<xsl:template name="Abs">
  <xsl:param name="TheValue"/>
  <xsl:variable name="Sign">
    <xsl:choose>
      <xsl:when test="$TheValue &lt; '0.0'">
        -1
      </xsl:when>
      <xsl:otherwise>
        1
      </xsl:otherwise>
    </xsl:choose> 
  </xsl:variable>

  <xsl:value-of select="number($Sign * $TheValue)"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- *************** Return the square root of a value ************** -->
<!-- **************************************************************** -->
<xsl:template name="Sqrt">
  <xsl:param name="num" select="0"/>       <!-- The number you want to find the square root of -->
  <xsl:param name="try" select="1"/>       <!-- The current 'try'.  This is used internally. -->
  <xsl:param name="iter" select="1"/>      <!-- The current iteration, checked against maxiter to limit loop count - used internally -->
  <xsl:param name="maxiter" select="10"/>  <!-- Set this up to insure against infinite loops - used internally -->

  <!-- This template uses Sir Isaac Newton's method of finding roots -->

  <xsl:choose>
    <xsl:when test="$try * $try = $num or $iter &gt; $maxiter">
      <xsl:value-of select="$try"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="Sqrt">
        <xsl:with-param name="num" select="$num"/>
        <xsl:with-param name="try" select="$try - (($try * $try - $num) div (2 * $try))"/>
        <xsl:with-param name="iter" select="$iter + 1"/>
        <xsl:with-param name="maxiter" select="$maxiter"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


</xsl:stylesheet>